/*
 * GraphsMenuOp.java
 *
 * Created on March 16, 2003, 9:00 PM
 */

/**
 *
 * @author  Amar
 */


/*
 *The graphs implemented in this file were done using the packages provided by JFreeChart.
 *We thank them for making there source files available.
 *JFREECHART website:
 *http://www.jfree.org/jfreechart/index.html
 *
 *JFREECHART API:
 *http://www.object-refinery.com/jfreechart/javadoc/index.html
 *
 */


import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JTable;

import com.jrefinery.chart.ChartFactory;
import com.jrefinery.chart.JFreeChart;
import com.jrefinery.data.AbstractDataset;
import com.jrefinery.data.DefaultCategoryDataset;
import com.jrefinery.data.DefaultPieDataset;
import com.jrefinery.data.XYSeries;
import com.jrefinery.data.XYSeriesCollection;


/**
 * Performs the functions needed to graph the data in the spreadsheet.
 * @author Spring 2004
 * @version 2.1
 */ 
public class GraphsMenuOp implements ClipboardOwner {

	/**
		  * The Table
		  */
		 private JTable table;
		 /**
		  *  Table Model
		  */
		 private UMTableModel tablemodel;
		 /**
		  *  The SpreadSheet
		  */
    private TerpSpreadSheet m_spreadsheet;
    

	/** The constant for pie graph */
	public static final int PIE = 0;
	/** The constant for column graph */
	public static final int COLUMN = 1;
	/** The constant for line graph */
	public static final int LINE = 2;
	/** The constant for row graph */
	public static final int ROW = 3;
    
	/**
 * Creates a new instance of GraphsMenuOp 
	 * @param spreadsheet - Input SpreadSheet
	 */
    public GraphsMenuOp( TerpSpreadSheet spreadsheet ) {
        
        m_spreadsheet = spreadsheet;
        table = spreadsheet.getTable();
        tablemodel = spreadsheet.getTableModel();
    }
	/**
	 * Makes a pie graph.
	 */
    public void pieGraph() //throws Exception
    {
        System.out.println( "PIE_GRAPH selected" );
        
        DefaultPieDataset pieDataset = null;
        
        AbstractDataset dataset = makeDatasetVertical( PIE , pieDataset );
        
        //Makes a call to the function which draws the graph in a frame.
        if ( dataset != null )/*FAULT:	if ( dataset == null )*/
        {
            JFreeChart chart = ChartFactory.createPie3DChart( "Pie", ( DefaultPieDataset )dataset , true, true, true );
            outputGraph( chart );
        }
    }
	/** Makes the dataset vertical.
		   * @param graphType the graph type
		   * @param dataset the dataset to make vertical
		   * @return returns an abstractdataset which is now vertical     
		   */
    private AbstractDataset makeDatasetVertical( int graphType , AbstractDataset dataset ) //Vertical represents labels are all down 1st column
    {
        
        
        if( table.getSelectedRowCount() != 0 && table.getSelectedColumnCount() > 1 ) //Must have at least 2 columns selected
        {
                        
            int startRow = table.getSelectedRow();
            int startCol = table.getSelectedColumn();
            int numrows = table.getSelectedRowCount() ;
            int numcols = table.getSelectedColumnCount();
            int endRow = startRow + numrows;
            int endCol = startCol + numcols;
           

		//System.out.println( "endCol  is: " + endCol );

            XYSeries xySeries;
            
            //Create Dataset of specific type for chart creation
            if ( graphType == PIE )
            {
               dataset = new DefaultPieDataset();
               endCol = startCol + 2;
            }
            else if ( graphType == ROW || graphType == COLUMN )
                dataset = new DefaultCategoryDataset();
            else //Line
            {
                dataset = new XYSeriesCollection();
                xySeries = new XYSeries("LINE");
                return makeLineDataset( (XYSeriesCollection)dataset , xySeries );
            }
            
            //Creates dataset for COLUMN, PIE, ROW charts but NOT!!! LINE 
            for ( int row = startRow; row < endRow ; row++ )
            {
                UMCell labelCell = tablemodel.getCellAt( row , startCol );
                UMCell valueCell;
                
                if ( labelCell.getValue() instanceof String ) //Label has to be a string
                {
                    for ( int col = startCol + 1 ; col < endCol ; col++ )
                    {
				//System.out.println("col: " + col );
                        valueCell = tablemodel.getCellAt( row , col );
                    
                        if ( !(valueCell.getValue() instanceof Number ) ) //Value of valueCell must be of type Number
                            return null;
                        if ( ((String)labelCell.getValue()).length() != 0 ) //Makes sure the string has something in it
                        {
                            String label = (String)labelCell.getValue();;
                            Object value = valueCell.getValue();
                            addToDataSet( label , value , dataset , row , col , graphType );
                        }
                        else
                        {
                            System.out.println( "ERROR" );
                            return null;
                        }
                    }
                }
                else return null;
            }
        }
        
        return dataset;
    }
    
    
    
    //Responsible for adding to any dataset that creates the four graphs
	/**
	   * Responsible for adding to any dataset that creates the four graphs.
		   * @param col  -col num
		   * @param value -Object value
		   * @param dataset -Dataset
		   * @param row - row num
		   * @param graphType -Type of Graph
		   * @param label - Label of the Graph
		   */
    private void addToDataSet( String label , Object value , AbstractDataset dataset , int row , int col , int graphType )
    {
        if ( graphType == PIE )
        {
            if ( value instanceof Integer )
                ((DefaultPieDataset)dataset).setValue( label , (Integer)value ); 
            else if ( value instanceof Float )
                ((DefaultPieDataset)dataset).setValue( label , (Float)value );
        }     
        else if ( graphType == ROW || graphType == COLUMN )
        {
            if ( value instanceof Integer )
                ((DefaultCategoryDataset)dataset).setValue( (Integer)value , label + "_" + col  , new Integer(row) ); 
            else if ( value instanceof Float )
                ((DefaultCategoryDataset)dataset).setValue( (Float)value , label + "_" + col , new Integer(row) );
        }
        
        
    }
    
	/**
			* This function makes a column graph.
			*/
    public void columnGraph() //throws Exception
    {
        System.out.println( "COLUMN_GRAPH selected" );
        DefaultCategoryDataset columnDataset = null;
        AbstractDataset dataset = makeDatasetVertical( COLUMN , columnDataset );
        
        //Makes a call to the function which draws the graph in a frame.
        if ( dataset != null )
        {
            JFreeChart chart = ChartFactory.createVerticalBarChart3D( "Column", "Domain" , "Range" , 
                                                                       ( DefaultCategoryDataset )dataset , true, true, true );
            outputGraph( chart );
        }     
        
        
    }
	/**
			* This function makes a line graph.
			*/
    public void lineGraph() //throws Exception 
    {
        System.out.println( "LINE_GRAPH selected" );
        
        XYSeriesCollection lineDataset = null;
        AbstractDataset dataset = makeDatasetVertical( LINE , lineDataset );
        
        //Makes a call to the function which draws the graph in a frame.
        if ( dataset != null )/*FAULT:	if ( dataset == null )*/
        {
            JFreeChart chart = ChartFactory.createLineXYChart( "LINE", "X" , "Y" , 
                                                                       ( XYSeriesCollection )dataset , true, true, true );
            outputGraph( chart );
        }     
    }
	/**
		 * This function makes a line data set add it the specified series which is added to the specified dataset.
		 * @param dataset the XYSeriesCollection to add the line dataset to
		 * @param xySeries the XYSeries where to add the line dataset that is made
		 * @return AbstractDataset - Returns the LineDataset
		 */
    private AbstractDataset makeLineDataset( XYSeriesCollection dataset , XYSeries xySeries )
    {
        int startRow = table.getSelectedRow();
        int startCol = table.getSelectedColumn();
        int numrows = table.getSelectedRowCount() ;
        int endRow = startRow + numrows;
        
        boolean error = false;
        
        for ( int row = startRow ; row < endRow; row++ )
        {
            UMCell XCell = tablemodel.getCellAt( row , startCol );
            UMCell YCell = tablemodel.getCellAt( row , startCol + 1 );
            
            if ( XCell.getType() != UMCell.TEXT && YCell.getType() != UMCell.TEXT ) //make sure they are valid cells
            {
                xySeries.add( (Number)XCell.getValue() , (Number)YCell.getValue()  );
            }
            else
                return null;
           
        }
        
        dataset.addSeries( xySeries );
        
        return dataset;
        
    }
	/** 
			* Creates a row graph.
			* 
			*/
    public void rowGraph() //throws Exception
    {
        System.out.println( "ROW_GRAPH selected" );
        
        DefaultCategoryDataset rowDataset = null;
        AbstractDataset dataset = makeDatasetVertical( ROW , rowDataset );
        
        //Makes a call to the function which draws the graph in a frame.
        if ( dataset != null )
        {
            JFreeChart chart = ChartFactory.createHorizontalBarChart3D( "ROW", "Range" , "Domain" , 
                                                                       ( DefaultCategoryDataset )dataset , true, true, true );
            outputGraph( chart );
        }       
    }
    
    //Takes a JFreeChart and outputs it in a frame
	/** 
			* Takes a JFreeChart and outputs it in a frame.
			* @param chart the chart to output in the frame
			*/
    private void outputGraph( JFreeChart chart ) //throws Exception
    {
            // write chart as JPEG file
            //ChartUtilities.saveChartAsJPEG(new File("chart.jpg"), chart, 500,300);

            final BufferedImage image = chart.createBufferedImage(500,300);

            JLabel lblChart = new JLabel();
            lblChart.setHorizontalAlignment(JLabel.CENTER);
            lblChart.setVerticalAlignment(JLabel.CENTER);
            ImageIcon icon = new ImageIcon( image );
            lblChart.setIcon( icon );

            JFrame frame = new JFrame( "TerpSpreadsheet " );
            
            JMenuBar menu = new JMenuBar();
            JMenu copy = new JMenu( "Copy" );
            
            
            JMenuItem copyToClipboard = new JMenuItem( "Copy to Clipboard" );
            copyToClipboard.addActionListener( new ActionListener() {
                public void actionPerformed( ActionEvent evt ) {
                    copyToClipboard( image );
                }});
     
            copy.add( copyToClipboard );    
            menu.add( copy );    
            frame.setJMenuBar( menu );

            frame.getContentPane().setLayout(new BorderLayout());
            frame.getContentPane().add(lblChart, BorderLayout.CENTER);
            frame.setSize(600,400);
            frame.setVisible(true);
   
            
    }
	/**
	   * Copies the image to the clipboard.
	   *
	   * @param image the image to copy to the clipboard
	   */ 
    private void copyToClipboard( BufferedImage image )
    {
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        
        Image graphImage;
        graphImage = (Image)image;
        
        UMClipObj clip = new UMClipObj(graphImage);
        
        clipboard.setContents(clip, this);
    }
	/**
			* Currently does nothing
			* @param transferable - Transferable ownershiip
			* @param clipboard -Clipboard
			*/
    public void lostOwnership(Clipboard clipboard, Transferable transferable) {
    }
    
}

    
