import javax.swing.undo.*;
import javax.swing.*;

/** The Paste Edit class record the changes occured to the
 * spreadsheet after performing a paste action. The paste edit support
 * undo / redo of paste action. Some of the code was borrowed from
 * Team Matrix.
 * @author TSS
	   * @version 3.0
 */
public class PasteUndo extends AbstractUndoableEdit {
      
    
	/**
	   * Old Starting Row Num
	   */
	  private int oldStartRow;
	  /**
	   *  Old number of rows
	   */
	  private int oldNumRows;
	  /**
	   * old Start Column
	   */
		  private int oldStartCol;
	  /**
	   * Number of old Columns
	   */
	  private int oldNumCols;
	  /**
	   *  Old Model
	   */
	  private UMTableModel oldModel;
	  /**
	   * holds current cell values
	   */
	  private Object oldStuff[][]; // holds current cell values
	  /**
	   * hold pasted cell values
	   */
    private Object pastedStuff[][]; // hold pasted cell values
    
    /** PasteUndo constructor.
     * Creates the temporary table to hold the values of each cell before the
     * cut operation takes place.
     *
     * @param jTabl the current table model
     * @param startRow the starting row index of selected range
     * @param numRows the number of rows selected
     * @param startCol the starting column index of the selected range
     * @param numCols the number of columns selected
     * @param model the table model prior to the clear operation */
    public PasteUndo(JTable jTabl, UMTableModel model, int startRow, int numRows, int startCol, int numCols) {
        oldModel = model;
        oldStartRow = startRow;/*FAULT:	oldStartRow = startCol;*/
        oldNumRows = numRows;
        oldStartCol = startCol;/*FAULT:	oldStartCol = startRow;*/
        oldNumCols = numCols;
        oldStuff = new Object[oldNumRows][oldNumCols];
        
        int newNumRows = oldStartRow+oldNumRows;
        int newNumCol = oldStartCol+oldNumCols;
        
        // will add rows or columns if clipboard exceeds boundaries
        // should we undo/redo additions of rows/cols?
        if ((oldModel.getRowCount() < newNumRows) || (oldModel.getColumnCount() < newNumCol)) {/*FAULT:	if ((oldModel.getRowCount() >= newNumRows) && (oldModel.getColumnCount() >= newNumCol)) {*/
            if (oldModel.getRowCount() < newNumRows)  {
                oldModel.setRowCount(newNumRows);
            }
            if (oldModel.getColumnCount() < newNumCol )  {
                oldModel.setColumnCount(newNumCol);
            }
            jTabl.setRowSelectionInterval(oldStartRow, oldStartRow+oldNumRows-1);
            jTabl.setColumnSelectionInterval(oldStartCol, oldStartCol+oldNumCols-1);
        }
        
        for (int i = 0; i < oldNumRows; i++)
            for (int j = 0; j < oldNumCols; j++)
                oldStuff[i][j] = ""+oldModel.getValueAt(oldStartRow + i, oldStartCol + j)+"";
        System.out.println(oldStartRow +" "+ oldNumRows +" "+ oldStartCol +" "+ oldNumCols +" "+ newNumRows +" "+ newNumCol);
    }
    
    /** This method repastes the values for each individual cell for the
     * selected range.
     */
    public void addRedoEdit() {
        pastedStuff = new Object[oldNumRows][oldNumCols];
        for (int i = 0; i < oldNumRows; i++)
            for (int j = 0; j < oldNumCols; j++)
                pastedStuff[i][j] = ""+oldModel.getValueAt(oldStartRow + i, oldStartCol + j)+"";
    }
    
    /** The actual undo function to put back the values of each cell.
     *
     * @throws CannotUndoException if undo is unavailable
     */
    public void undo() throws CannotUndoException {
        for (int i = 0; i < oldNumRows; i++)
            for (int j = 0; j < oldNumCols; j++)
                oldModel.setValueAt(oldStuff[i][j],oldStartRow + i, oldStartCol + j);
    }
    
    /** The actual redo function to clear the selected cells.
     *
     * @throws CannotRedoException if redo is unavailable
     */
    public void redo() throws CannotRedoException {
        for (int i = 0; i < oldNumRows; i++)
            for (int j = 0; j < oldNumCols; j++)
                oldModel.setValueAt(pastedStuff[i][j],oldStartRow + i, oldStartCol + j);
    }
    
    /** Determines if undo can occur.
     *
     * @return true to allow undo
     */
    public boolean canUndo() { return true; }
    
    /** Determines if redo can occur.
     *
     * @return true to allow redo
     */
    public boolean canRedo() { return true; }
    
    /** Tells what operation the undo/redo is for.
     *
     * @return the name of the operation that occured
     */
    public String getPresentationName() { return "Paste"; }
}
