
import java.awt.Image;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;

/**
 * A Transferable which implements the capability required to transfer a
 * String.
 *
 * This Transferable properly supports <code>DataFlavor.stringFlavor</code>
 * and all equivalent flavors. Support for <code>DataFlavor.plainTextFlavor
 * </code> and all equivalent flavors is <b>deprecated</b>. No other
 * DataFlavors are supported.
 *
 *@author TSS
	   * @version 3.0
 * @see java.awt.datatransfer.DataFlavor.stringFlavor
 * @see java.awt.datatransfer.DataFlavor.plainTextFlavor
 */
public class UMClipObj implements Transferable, ClipboardOwner
 {
	/** Each instance represents the opaque concept 
	   of a data format as would appear on a clipboard, 
	   during drag and drop, or in a file system. */
    private static final DataFlavor[] flavors = {
        DataFlavor.stringFlavor,
	DataFlavor.plainTextFlavor,
        DataFlavor.imageFlavor
    };
	/** Used as data for clipboard */
		private String data;
    
		/** Raw text. */
		private String m_text;
    
		/** An Image Object. */
		private Image image;
	/**
	   * Creates a Transferable capable of transferring the specified String.
	   * @param fullText
	   * @param simpleText
	   */
    UMClipObj(String simpleText, String fullText){

         this.data = fullText;
         this.m_text = simpleText;    
    }
    
	/**
	  * Creates a Transferable capable of transferring a BufferedImage
	  * @param imageToClipboard
	  */
    UMClipObj(Image imageToClipboard){
        this.image = imageToClipboard;
    }

    /**
     * Returns an array of flavors in which this Transferable can provide
     * the data. <code>DataFlavor.stringFlavor</code> is properly supported.
     * Support for <code>DataFlavor.plainTextFlavor</code> is
     * <b>deprecated</b>.
     *
     * @return an array of length two, whose elements are <code>DataFlavor.
     *         stringFlavor</code> and <code>DataFlavor.plainTextFlavor</code>.
     */
    public DataFlavor[] getTransferDataFlavors() {
        // returning flavors itself would allow client code to modify
        // our internal behavior
	return (DataFlavor[])flavors.clone();
    }

    /**
 * Returns whether the requested flavor is supported by this Transferable.
     *
     * @param flavor the requested flavor for the data 
     * @return true if <code>flavor</code> is equal to
     *   <code>DataFlavor.stringFlavor</code> or
     *   <code>DataFlavor.plainTextFlavor</code>; false if <code>flavor</code>
     *   is not one of the above flavors
     */
    public boolean isDataFlavorSupported(DataFlavor flavor) {
        for (int i = 0; i < flavors.length; i++) {
	    if (flavor.equals(flavors[i])) {/*FAULT:	if (!(flavor.equals(flavors[i]))) {*/
	        return true;/*FAULT:	return false;*/
	    }
	}
	return false;
    }
    
    /**
 * Returns the Transferable's data in the requested DataFlavor if
     * possible. If the desired flavor is <code>DataFlavor.stringFlavor</code>,
     * or an equivalent flavor, the String representing the selection is
     * returned. If the desired flavor is </code>DataFlavor.plainTextFlavor
     * </code>, or an equivalent flavor, a Reader is returned. <b>Note:<b>
     * The behavior of this method for </code>DataFlavor.plainTextFlavor</code>
     * and equivalent DataFlavors is inconsistent with the definition of
     * <code>DataFlavor.plainTextFlavor</code>.
     *
     * @param flavor the requested flavor for the data
     * @return the data in the requested flavor, as outlined above.
     * @throws UnsupportedFlavorException if the requested data flavor is
     *         not equivalent to either <code>DataFlavor.stringFlavor</code>
     *         or <code>DataFlavor.plainTextFlavor</code>.
     * @throws IOException if an IOException occurs while retrieving the data.
     *         By default, StringSelection never throws this exception, but a
     * @see java.io.Reader
     */
    public Object getTransferData(DataFlavor flavor)
        throws UnsupportedFlavorException, IOException
    {
	if (flavor.equals(DataFlavor.stringFlavor)) {/*FAULT:	if (!(flavor.equals(DataFlavor.stringFlavor))) {*/
            // returns clipboard data without font formats
            // used when pasting from other applications into TerpSpreadsheet (and vice versa)
            return (Object)m_text;
            
	} else if (flavor.equals(DataFlavor.plainTextFlavor)) {/*FAULT:	} else if (!(flavor.equals(DataFlavor.plainTextFlavor))) {*/
            // returns copied data with font formats attached
            // used when pasting within TerpSpreadSheet
            return (Object)data;
        } else if (flavor.equals(DataFlavor.imageFlavor)){
            return (Object)image;
	} else {
            System.out.println("Exception: Attempting to access clipboard with unsupported DataFlavor");
	    throw new UnsupportedFlavorException(flavor);
	}
    }
    
    /** Method required for implementing clipboard.
     *
     * @param clipboard current clipboard
     * @param transferable ??
     */    
    public void lostOwnership(Clipboard clipboard, 
                              Transferable transferable) 
    { 
        System.out.println ("Lost Clipboard ownership"); 
    } 
} 




