//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//CMSC435  UMDialog (some of the code is adapted from
//                        SharpTools project )
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**************************************************************
 * Class: UMDialog
 * Description: this is the class that provides a customized dialog
 *             for our project
 * Disclaimer: Some of the code and and ideas we found online in
 *             the ShartTools project.  We thank them for making
 *             their code availible.
 *
 * @author Nora Husain
 * @version 1.0
 * @see JDialog
 *
 * This class runs on a PC with Java version 1.4.1.
 * /*************************************************************
 */
class UMDialog extends JDialog {
    
    
//	Constants
	 /** closed option value */
	 static final public int CLOSED_OPTION = JOptionPane.CLOSED_OPTION;
	 /** yes option value */
	 static final public int YES_OPTION = JOptionPane.YES_OPTION;
	 /** no option value */
	 static final public int NO_OPTION = JOptionPane.NO_OPTION;
	 /** okay option value */
	 static final public int OK_OPTION = JOptionPane.OK_OPTION;
	 /** cancel option value */
	 static final public int CANCEL_OPTION = JOptionPane.CANCEL_OPTION;
    
	/** store the ok image */
		static final private ImageIcon okIcon =
	TerpSpreadSheet.getImageIcon("ok.gif");
		/** store the cancel image */
		static final private ImageIcon cancelIcon =
	TerpSpreadSheet.getImageIcon("cancel.gif");
		/** store the no image */
		static final private ImageIcon noIcon =
	TerpSpreadSheet.getImageIcon("no.gif");
        
	/** create the ok button object */
		final private JButton okButton =
		new JButton(UIManager.get("OptionPane.okButtonText").toString(),
		okIcon);
		/** create the cancel button object */
		final private JButton cancelButton =
		new JButton(UIManager.get("OptionPane.cancelButtonText").toString(),
		cancelIcon);
		/** create the yes button object */
		final private JButton yesButton =
		new JButton(UIManager.get("OptionPane.yesButtonText").toString(),
		okIcon);
		/** create the no button object */
		final private JButton noButton =
		new JButton(UIManager.get("OptionPane.noButtonText").toString(),
		noIcon);
    
		/** store the array for an ok button */
		final private Object[] ok = { okButton };
		/** store the array for an yes and no buttons */
		final private Object[] yesno = { yesButton, noButton };
		/** store the array for an yes, no, and cancel buttons */
		final private Object[] yesnocancel = { yesButton, noButton, cancelButton };
		/** store the array for the ok and cancel buttons */
		final private Object[] okcancel = { okButton, cancelButton };
		/** store the CLOSED_OPTION value */
		private int choice = CLOSED_OPTION;
    
    
    // override all constructors
	/**
	   * UMDialog constructor
	   *
	   * Description: create a non-modal dialog
	   *
	   * Parameter for constructor:
	   * 
	   * @param c  parent dialog
	   */
    UMDialog(Dialog c) {
        super(c);
    }
    
	/**
	* UM Dialog constructor
	*
	* Parameter for constructor:
	*
	* @param c  parent dialog 
	* @param modal modal whether to create a modal dialog
	*/
    UMDialog(Dialog c, boolean modal) {
        super(c, modal);
    }
    
	/**
	 * UMDialog constructor
	 *
	 * create a non-modal dialog
	 *
	 * Parameters for constructor:
	 * @param c parent dialog 
	 * @param title String 
	 */
    UMDialog(Dialog c, String title) {
        super(c, title);
    }
    
	/** UMDialog constructor
 
	* @param title parent dialog
	* @param c parent dialog
	* @param modal whether to create a modal dialog
	*/
    UMDialog(Dialog c, String title, boolean modal) {
        super(c, title, modal);
    }
    
	/**
	  * UMDialog constructor
	  *
	  * create a non-modal dialog
	  *
	  *  @param  c parent frame
	  */
    UMDialog(Frame c) {
        super(c);
    }
    
	/**
	 * UMDialog constructor
	 *
	 *  @param  c parent frame
	 *  @param  modal boolean whether to create a modal dialog
	 */
    UMDialog(Frame c, boolean modal) {
        super(c, modal);
    }
    
	/**
	 * UMDialog constructor
	 *
	 * Description: create a non-modal dialog
	 *
	 * @param  c parent frame 
	 * @param  title String
	 */
    UMDialog(Frame c, String title) {
        super(c, title);
    }
    
	/**
	 * UMDialog constructor
	 * @param c parent frame
	 * @param   title String
	 * @param  modal whether to create a modal dialog
	 */
    UMDialog(Frame c, String title, boolean modal) {
        super(c, title, modal);
    }
    
    /**
     * Protected Method:  createRootPane
     *
     * Description: override this protected function to register ESCAPE with the root pane
     *
     * @return a customized JRootPane object
     */
    protected JRootPane createRootPane() {
        KeyStroke stroke = KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0);
        JRootPane rootPane = new JRootPane();
        rootPane.registerKeyboardAction(
        new ActionListener() {
            public void actionPerformed
            (ActionEvent ev) {
                if (onClose()) {
                    choice = CLOSED_OPTION;
                    setVisible(false);
                }
            }
        },
        stroke,
        JComponent.WHEN_IN_FOCUSED_WINDOW);
        return rootPane;
    }
    
    /**
     * Previous author provides several hooks for the sublasses to override..
     */
    
    /**
     * Protected Method: onOpen
     *
     * Description: This function is called when the dialog window is opened.
     *
     */
    protected void onOpen() {
        
    }
    
    /**
     * Protected method: onClose
     *
     * Description:
     * This function is called when
     * (1) the close button is clocked
     * (2) Escape is pressed
     *
     * By default, we think it's the same as clicking Cancel button
     *
     * @return whether the dialog should be closed
     */
    protected boolean onClose() {
        return onCancel();
    }
    
    /**
     * Protected Method: onOK
     *
     * Description: This function is called when the OK button is pressed
     *
     * @return whether the dialog should be closed
     */
    protected boolean onOK() {
        return true;
    }
    
    /**
     * Protected Method: onCancel
     *
     * Desciption: This function is called when the Cancel button is pressed
     *
     * @return whether the dialog should be closed
     */
    protected boolean onCancel() {
        return true;
    }
    
    /**
     * Protected Method: onYes
     *
     * Description: This function is called when the Yes button is pressed
     *
     * @return whether the dialog should be closed
     */
    protected boolean onYes() {
        return true;
    }
    
    /**
     * Protected Method: onNo
     *
     * Description: This function is called when the No button is pressed
     *
     * @return whether the dialog should be closed
     */
    protected boolean onNo() {
        return true;
    }
    
    /** Protected Method: setOptionPane
     *
     * Description: This is used by subclasses to set the JOptionPane component
     * in the dialog.
     *
     * @param message - the message box
     * @param messageType - Type the message type
     * @param optionType - the option type
     * @param icon - icon the icon to display
     */
    protected void setOptionPane(Object message,
    int messageType,
    int optionType,
    Icon icon) {
        
        setOptionPane(message, messageType, optionType, icon, 0);
    }
    
    /**
     * Protected method: setOptionPane
     *
     * Description: This is used by subclasses to set the JOptionPane component
     * in the dialog.
     *
     * @param message -the message box
     * @param messageType -the message type
     * @param optionType -the option type
     * @param icon -the icon to display
     * @param defaultIndex -the default index
     */
    protected void setOptionPane(Object message,
    int messageType,
    int optionType,
    Icon icon,
    int defaultIndex) {
        
        addWindowListener(new WindowAdapter() {
            
            public void windowOpened(WindowEvent we) {
                // set the initial focus to rowField
                onOpen();
            }
            
            public void windowClosing(WindowEvent we) {
                if (onClose()) {
                    choice = CLOSED_OPTION;
                    setVisible(false);
                }
            }
        });
        
        // handle this by onClose()
        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        
        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (onOK()) {
                    choice = OK_OPTION;
                    setVisible(false);
                }
            }
        });
        
        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (onCancel()) {
                    choice = CANCEL_OPTION;
                    setVisible(false);
                }
            }
        });
        
        yesButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (onYes()) {
                    choice = YES_OPTION;
                    setVisible(false);
                }
            }
        });
        
        noButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                if (onNo()) {
                    choice = NO_OPTION;
                    setVisible(false);
                }
            }
        });
        
        // setup Mnemonic
        okButton.setMnemonic(KeyEvent.VK_O);
        cancelButton.setMnemonic(KeyEvent.VK_C);
        yesButton.setMnemonic(KeyEvent.VK_Y);
        noButton.setMnemonic(KeyEvent.VK_N);
        
        Object[] buttons = setButtons(optionType);
        JOptionPane optionPane = new JOptionPane(message, messageType,
        optionType, icon,
        buttons,
        buttons[defaultIndex]);
        
        setContentPane(optionPane);
        
        pack();
        setLocationRelativeTo(getParent());
    }
    
     /**
     * Private method : setButtons
     *
     * @param type - int
     * set button array
     * @return Object[]
     */
    private Object[] setButtons(int type) {
        
        Object[] buttons;
        
        switch (type) {
            case JOptionPane.YES_NO_CANCEL_OPTION:
                buttons = yesnocancel;
                break;
            case JOptionPane.YES_NO_OPTION:
                buttons = yesno;
                break;
            case JOptionPane.OK_CANCEL_OPTION:
                buttons = okcancel;
                break;
            default:
                buttons = ok;/*FAULT:	buttons = yesno;*/
                break;
        }
        
        return buttons;
    }
    
    /**
     * Public method: getChoice
     *
     * @return user choice- int
     */
    public int getChoice() {
        return choice;
    }
    /**
     * Public method: isCancelled
     *
     * @return whether this operation was cancelled- boolean - true or false
     */
    public boolean isCancelled() {
        return choice == CANCEL_OPTION || choice == CLOSED_OPTION;
    }
}




