//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//CMSC435  UMOptionPane (some of the code is adapted from
//                        SharpTools project )
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.Label;

import javax.swing.Icon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Class: UMOptionPane --
 * Description: this is the class that allows to show different
 *             kind of dialogs
 * Disclaimer: Some of the code and and ideas we found online in
 *            the ShartTools project.  We thank them for making
 *            their code availible.
 * @author TSS
 * @version  3.0
 */
public class UMOptionPane {
    
    /** Shows Input dialog to the user.
     *
     * @param parentComponent the parent Component for the dialog
     * @param message the Object to display
     * @param title the String to display in the dialog title bar
     * @param messageType the type of message to be displayed
     * @param icon the Icon image to display
     * @param selectionValues an array of Objects that gives the possible selections
     * @param initialSelectionValue the value used to initialize the input field
     * @return current object
     */
    public static Object showInputDialog(Component parentComponent,
    Object message,
    String title,
    int messageType,
    Icon icon,
    Object[] selectionValues,
    Object initialSelectionValue) {
        return showInputDialog(parentComponent, message, title, messageType,
        icon, selectionValues, initialSelectionValue,
        false);
    }
    
    /** Gets integer.
     *
     * @param parentComponent the parent Component for the dialog
     * @param message the Object to display
     * @param title the String to display in the dialog title bar
     * @param messageType the type of message to be displayed
     * @param icon the Icon image to display
     * @param initialSelectionValue the value used to initialize the input field
     * @return the integer
     */
    public static Integer showIntegerInputDialog(Component parentComponent,
    Object message,
    String title,
    int messageType,
    Icon icon,
    Object initialSelectionValue) {
        Object input = showInputDialog(parentComponent, message, title,
        messageType, icon, null,
        initialSelectionValue, true);
        try {
            if (input != null)
                return new Integer(Integer.parseInt(input.toString()));
        }
        catch (Exception e) {
            if (input.toString().trim().length() == 0)
                return null;
            return new Integer(-1);
            
        }
        
        return null;
    }
    
    /** Shows a float input dialog to the user.
     *
     * @param parentComponent the parent Component for the dialog
     * @param message the Object to display
     * @param title the String to display in the dialog title bar
     * @param messageType the type of message to be displayed
     * @param icon the Icon image to display
     * @param initialSelectionValue the value used to initialize the input field
     * @return the float value
     */
    public static Float showFloatInputDialog(Component parentComponent,
    Object message,
    String title,
    int messageType,
    Icon icon,
    Object initialSelectionValue) {
        Object input = showInputDialog(parentComponent, message, title,
        messageType, icon, null,
        initialSelectionValue, true);
        
        try {
            if (input != null)
                return new Float(Float.parseFloat(input.toString()));
        }
        catch (Exception e) {
            if (input.toString().trim().length() == 0)
                return null;
            return new Float(-1);
            
        }
        
        return null;
    }
    
    
	  
    /**
     * This is used by showInputDialog and showIntegerInputDialog
     *
     * @param parentComponent the parent Component for the dialog
     *@param message the Object to display
     * @param title the String to display in the dialog title bar
     *@param messageType the type of message to be displayed
     * @param icon the Icon image to display
     * @param selectionValues an array of Objects that gives the possible selections
     *@param initialSelectionValue the value used to initialize the input field
     *@param isInteger whether we only allow integer input
     * @return Object
     */
    private static Object showInputDialog(Component parentComponent,
    Object message,
    String title,
    int messageType,
    Icon icon,
    Object[] selectionValues,
    Object initialSelectionValue,
    boolean isInteger) {
        JPanel panel = new JPanel(new BorderLayout());
        JComponent com;
        int choice;
        
        if (initialSelectionValue == null)
            initialSelectionValue = new String("");
        if (selectionValues == null) {
            // we use a JTextField
            if (isInteger) {
                com = new NumberField(true, true);
                ((JTextField)com).setText(initialSelectionValue.toString());
            }
            else
                com = new JTextField(initialSelectionValue.toString());
            
            ((JTextField)com).selectAll();
        }
        else {
            JComboBox box = new JComboBox(selectionValues);
            if (initialSelectionValue != null)
                box.setSelectedItem(initialSelectionValue);
            else
                box.setSelectedIndex(0);
            
            com = box;
            
        }
        
        if (message instanceof Component)
            panel.add((Component)message, BorderLayout.NORTH);
        else
            panel.add(new Label(message.toString()), BorderLayout.NORTH);
        
        panel.add(com, BorderLayout.CENTER);
        
        choice = showOptionDialog(parentComponent, panel, title,
        JOptionPane.OK_CANCEL_OPTION,
        messageType, icon);
        
        if (choice == JOptionPane.OK_OPTION) {
            if (com instanceof JTextField)
                return ((JTextField)com).getText();
            else
                return ((JComboBox)com).getSelectedItem();
        }
        else
            return null;
    }
    
    /** Shows a message dialog to the user.
     *
     * @param parentComponent the parent component
     * @param message the message to be displayed
     */
    public static void showMessageDialog(Component parentComponent,
    Object message) {
        showMessageDialog(parentComponent, message, null, 0, null);
    }
     
    /** Shows a message dialog to the user.
     *
     * @param parentComponent the parent component
     * @param message the message to be displayed
     * @param title dialog title
     * @param messageType the message type
     */
    public static void showMessageDialog(Component parentComponent,
    Object message,
    String title,
    int messageType) {
        showMessageDialog(parentComponent, message, title, messageType, null);
    }
    
    /** Shows a message dialog to the user.
     *
     * @param parentComponent the parent component
     * @param message the message to be displayed
     * @param title dialog title
     * @param messageType the message type
     * @param icon the icon to display
     */
    public static void showMessageDialog(Component parentComponent,
    Object message,
    String title,
    int messageType,
    Icon icon) {
        showOptionDialog(parentComponent, message, title,
        JOptionPane.DEFAULT_OPTION,
        messageType, icon, 0);
    }
    
    /** Shows option dialog to the user.
     *
     * @param parentComponent the parent component
     * @param message he message to be displayed
     * @param title dialog title
     * @param optionType the option type
     * @param messageType the message type
     * @param icon the icon to display
     * @return choice
     */
    public static int showOptionDialog(Component parentComponent,
    Object message,
    String title,
    int optionType,
    int messageType,
    Icon icon) {
        return showOptionDialog(parentComponent, message, title, optionType,
        messageType, icon, 0);
    }
    
    /** Note the interface is different from JOptionPane.showOptionDialog()
     *
     * @param parentComponent the parent component
     * @param message the message to be displayed
     * @param title dialog title
     * @param optionType the option type
     * @param messageType the message type
     * @param icon the icon to display
     * @param defaultIndex defaultIndex
     * @return choice
     */
    public static int showOptionDialog(Component parentComponent,
    Object message,
    String title,
    int optionType,
    int messageType,
    Icon icon,
    int defaultIndex) {
        
        UMDialog dialog;
        
        if (parentComponent instanceof Frame)
            dialog = new UMDialog((Frame)parentComponent, title, true);
        else
            dialog = new UMDialog((Dialog)parentComponent, title, true);
        
        dialog.setOptionPane(message, messageType,
        optionType, icon,
        defaultIndex);
        
        dialog.show();
        return dialog.getChoice();
    }
    
}

