import java.util.Iterator;
import java.util.LinkedList;

import javax.swing.JOptionPane;
import javax.swing.JTable;


/**
 * The User Defined Functions class, here the user can define Functions types that then are read
 * into the Spread Sheet application. Procedure
 * 1. Extend the Function class for the actuall function
 * 2. Register the function with the SpreadSheet
 * 3. Use the FillSelection function to populate the user function
 * 
 * @author TSS
 * @version 3.0
 */

public class UserFunctions {
	
	/**
	  * The Jtable that will be derived form the UMSpreadSheet passed in
	  */
	 static private JTable table;
	 /**
	  * The table model that will be derived form the UMSpreadSheet passed in
	  */
	 static private UMTableModel tablemodel;
    
	/**
      Example of a user defined function
      POW takes the spreadsheet and registers the UserFunctionPOW() class
      Fills the selection from the spreadsheet
      @param m_spreadsheet - the spreadsheet
      @return (Number) null
		  */
		public static Number POW(TerpSpreadSheet m_spreadsheet)
	{
		System.out.println("Usr POW");
		Formula.addregisterFunctions("POW", new UserFunctionPOW());
		FillSelection("POW",m_spreadsheet);
		Number out=null;
		return out;
	}
	/**
	  Example of a user defined function
	  STDEV takes the spreadsheet and registers the UserFunctionSTDEV() class
	  Fills the selection from the spreadsheet
	  @param m_spreadsheet - the spreadsheet
	  @return (Number) null
		  */
	public static Number STDEV(TerpSpreadSheet m_spreadsheet)
	{
		System.out.println("Usr STDEV");
		Formula.addregisterFunctions("STDEV", new UserFunctionSTDEV());
		FillSelection("STDEV",m_spreadsheet);
		Number out=null;
		return out;
	}




	/**
	 * Takes the function name/spreadsheet and generates the
	 * '=FUNCTION_NAME( .........) ' in the celected cell
	 * @param m_spreadsheet -the spreadsheet
	 * @param functionName -The function name
	 */
	private static void FillSelection(String functionName,TerpSpreadSheet m_spreadsheet )
	{
		System.out.println("User Function: "+ functionName);

		table=m_spreadsheet.getTable();
		tablemodel=m_spreadsheet.getTableModel();
				if(table.getSelectedRowCount() != 0){
			String text;
			CellRange range= new CellRange(table.getSelectedRows(), table.getSelectedColumns());
			StringBuffer buff=new StringBuffer();
			int startRow=table.getSelectedRow();
			int startCol=table.getSelectedColumn();
			int numrows=table.getSelectedRowCount();
			int numcols=table.getSelectedColumnCount();
			Number curr=null;
		   try{
			curr=  tablemodel.getNumericValueAt(startRow+numrows-1,startCol+numcols-1);
		   System.out.println("Curr val: "+tablemodel.getNumericValueAt(startRow+numrows-1,startCol+numcols-1));
		   }catch (Exception e){e.printStackTrace();}

			String temp=new String();
			temp = temp + ("="+functionName+"(");
			temp = temp+ curr.toString();
			int x=0,y=0,count =1;

			//Calculating
			for(x=0; x<numcols; x++)
				for(y=0;y<numrows;y++) {
					if(x!=numcols-1 || y!=numrows-1){
						if(count != 0)
							temp = temp+ ",";
						char tempChar = 'A';
						tempChar += (startCol + x -1);
						temp += tempChar  + (Integer.toString(y+startRow+1));
						System.out.println(tempChar);
						count++;

					}
				}
			temp += ")";
			System.out.println("Created: "+temp.toString() + "count " + count);

			if (count <=1)
				tablemodel.doSetValueAt(temp,startRow+numrows-1,startCol+numcols-1);
			else{
				UMOptionPane.showMessageDialog(m_spreadsheet,
			   "Too Many Cells are Selected !!!",
			   "Square Root Operation",
			   JOptionPane.WARNING_MESSAGE,
			   null);
			}
		}

	}

}
    
 // UserFunctions


/**
 * Class: UserFunctionPOW
 *
 * Description: This will find the power of the first element to the second.
 * It gets the values from UMTableModel.
 *
 * <p> Here's an example: <br>
 *
 * <code>POW</code><br>
 *   usage: <code>=POW(base,power)</code><br>
 *   
 * @author TSS
 * @version 3.0
 */
class UserFunctionPOW extends Function {

    /**
     * Finds the POWER of the value at the specified row and column.
     *
     * @param table the actual values to sum
     * @param node a LinkedList
     * @param row the int row coordinate
     * @param col the int column coordinate
     * @return Number the value calculated
     * @throws ParserException throws an exception which contains the parser
     */    
    public Number evaluate(UMTableModel table, Node node,
			  int row, int col) throws ParserException {

   int operation = 0;

	// requires parameters
	checkParamsExist(node);
	float value=0;
	int count=0;
	Float base=null;
	Float exp=null;
	
	LinkedList params = node.getParams();

	if ((params != null )&& (params.size() == 2)) {

	    Iterator it = params.iterator();
	    while (it.hasNext()) {
		Node nd = (Node)it.next();
		if (isRange(nd)) {
			System.out.println("EXPT");
     	   throw new ParserException("#PARAM?");
		}
		else {
		     value =
			Formula.evaluate(table, nd.getExp(), row, col).floatValue();
			if (count==0)
			{ base= new Float(value);
			count++;		
			}
			else
			{exp= new Float(value);
			}
	     }
		}
	 }
	else
	{
				System.out.println("EXPT");
	     	   throw new ParserException("#PARAM?");
	 
	}
		Double Dbase = new Double ( base.toString());
		Double Dexp = new Double ( exp.toString());
        value = (float) Math.pow(Dbase.doubleValue() ,Dexp.doubleValue());	
	return new Float(value);
    }
	
		

//	}
    
//	return new Float(value);
  //   }

    /**
     * Gets what this function is doing
     *
     * @return a string */      
    public String getUsage() {
	return "POW(base,power)";
    }

    /**
     * Gets the description of this function
     *
     * @return a string */      
    public String getDescription() {
	return "Returns a power of a number.";
    }
}

/**
 * Class: UserFunctionSTDEV
 *
 * Description: This will find the standard deviation of the range
 * It gets the values from UMTableModel.
 *
 * <p> Here's an example: <br>
 *
 * <code>STDEV</code><br>
 *   usage: <code>=STDEV(value,value,...)</code><br>
  * @author TSS
 * @version 3.0  
 */


class UserFunctionSTDEV extends Function {

    /**
     * Finds the Standard Deviation of the range 
     *
     * @param table the actual values to sum
     * @param node a LinkedList
     * @param row the int row coordinate
     * @param col the int column coordinate
     * @return Number the value calculated
     * @throws ParserException throws an exception which contains the parser
     */    
    

    
    public Number evaluate(UMTableModel table, Node node,
			  int row, int col) throws ParserException {

   int operation = 0;

	// requires parameters
	checkParamsExist(node);
	float value=0;
	float average=0;
	float nCells=0;
	float dev_sum=0;
	
	
	LinkedList params = node.getParams();

	if (params != null) {
		average = (new FunctionAverage()).evaluate(table, node, row, col).floatValue();
		System.out.println("Average: "+ average);
		nCells = (new FunctionCount()).evaluate(table, node, row, col).floatValue();
		System.out.println("Ncells: "+ nCells);
	    Iterator it = params.iterator();

	    while (it.hasNext()) {
		// the first parameter
		Node exp = (Node)it.next();

		if (isRange(exp)) {
		    // if it's a range then count the number of cells
		   CellPoint[] addr =
			getFirst(exp).getAddressRange(row, col);
		    // for a range, go over the whole range
		    for (int i = addr[0].getRow(); i <= addr[1].getRow(); i++)
			for (int j = addr[0].getCol(); j <= addr[1].getCol(); j++)
			    // get the numeric value of that cell
			    dev_sum +=(float) Math.pow((double)(table.getNumericValueAt(i, j).floatValue()-average),2);
				System.out.println("After range the dev_sum: "+ dev_sum);
		}
		else {
		    // otherwise count one
		    value =	Formula.evaluate(table, exp.getExp(), row, col).floatValue();	
		    dev_sum += (float) Math.pow((double) (value-average),2);
		}
	    }

	}
	else
	{
				System.out.println("EXPT");
	     	   throw new ParserException("#PARAM?");
	 
	}
		System.out.println("After loops the dev_sum: "+ dev_sum);
	if( (nCells!=0) && (dev_sum!=0))
	{	dev_sum=dev_sum/(nCells-1);
		value=(float) Math.sqrt(dev_sum);
	}	
	return new Float(value);
    }
	
		

//	}
    
//	return new Float(value);
  //   }

    /**
     * Gets what this function is doing
     *
     * @return a string */      
    public String getUsage() {
	return "STDEV(value,value,...)";
    }

    /**
     * Gets the description of this function
     *
     * @return a string */      
    public String getDescription() {
	return "Returns a Standard Deviation of a Range.";
    }
}