import java.util.*;
import java.awt.*;
import java.awt.Color;
import javax.swing.border.*;
import javax.swing.UIManager;
/**
 * Class: UMCell
 * Description: This is a wrapper class for all objects in the table.
 *
 *    A cell has a value which is either text or a number.
 *    Formula support will be added later.
 * @author TSS
	   * @version 3.0
 */
public class UMCell {
    /** The integer code that denotes the cell holds text. */
    public static final int TEXT = 0;
    
    /** The integer code that denotes the cell holds numbers. */
    public static final int NUMBER = 1;
    
    /** The integer code that denotes the cell holds a formula. */
    public static final int FORMULA = 2;
    
    
	/** Value of the cell. In a formula, value holds the evaluated value. */
  private Object m_value;
  /** store the font value */
  private Font m_font;
  /** store the background color */
  private Color m_bColor;
  /** store the font color */
  private Color m_fColor;
  /** store the border */
  private Border m_border;
    
    
    /**
     * This expression contains the string and internal representation
     * of the formula if it exists.
     */
    private Formula m_expression;
	/** store the TreeSet object */
    private TreeSet m_refs;
    
    /**
     * Public Constructor: UMCell
     * Description: this creates an empty cell
     */
    public UMCell() {
        m_value = null;
        m_font = null;
        m_bColor = null;
        m_fColor = null;
        m_border = null;
        m_expression = null;
        m_refs = null;
    }
    
    /** Public Copy Constructor: UMCell
     * Description: this creates a new cell and copies
     *              info from the cell passed in
     * @param cell current cell
     */
    public UMCell(UMCell cell) { copy(cell); }
    
    /** Public Constructor: UMCell
     * Description: This constructor constructs a cell that will
     *             not have cells referencing it or fromulas.
     *             Basically only used for creating the row and
     *             column labels.
     *
     * @param datum current data
     */
    public UMCell(Object datum) {
        m_value = datum;
        m_font = null;
        m_bColor = null;
        m_fColor = null;
        m_border = null;
        m_expression = null;
        m_refs = null;
    }
    
    /** Public Constructor: UMCell
     * Description: This constructor constructs a cell that will
     *             not have cells referencing it or fromulas.
     *             Basically only used for creating the row and
     *             column labels.
     *
     * @param datum the value of the "label" cell (not a Formula)
     * @param font font for the cell
     * @param bColor background color for the cell
     * @param fColor forground color for the cell
     * @param border border for the cell
     */
    public UMCell(Object datum, Font font, Color bColor, Color fColor, Border border) {
        m_value = datum;
        m_font = font;
        m_bColor = bColor;
        m_fColor = fColor;
        m_border = border;
        m_expression = null;
        m_refs = null;
    }
    /** Public Constructor: UMCell
     * Description: The constructor creates a new data cell which
     *             is not a formula.
     *
     * @param datum the value of the cell (text or number)
     * @param reference the list of cells that reference this one
     *                 (can be null) THIS IS NOT USED NOW
     */
    public UMCell(Object datum, TreeSet reference) {
        m_refs = reference;
        m_value = datum;
        m_expression = null;
        m_font = null;
        m_bColor = null;
        m_fColor = null;
        m_border = null;
    }
    /** Public method: copy
     * Description: This copies values of cell, into cell
     *
     * NOTE: here and later add to make copy of data
     * @param cell current cell
     */
    public void copy(UMCell cell) {
        m_value = null;
        m_font = null;
        m_bColor = null;
        m_fColor = null;
        m_border = null;
        m_expression = null;
        m_refs = null;
        
        if (cell.m_expression != null)
            setFormula(cell.getFormula());
        
        if (cell.m_value != null) {
            if (cell.m_value instanceof Integer)
                m_value = new Integer(((Integer)cell.m_value).intValue());
            else if (cell.m_value instanceof Float)
                m_value = new Float(((Float)cell.m_value).floatValue());
            else
                m_value = new String(cell.m_value.toString());
        }
        
        if (cell.m_font != null)
            m_font = new Font(cell.m_font.getAttributes());
        if (cell.m_bColor != null)
            m_bColor = new Color(cell.m_bColor.getRGB());
        if (cell.m_fColor != null)
            m_fColor = new Color(cell.m_fColor.getRGB());
        
        if (cell.m_border != null) {
            m_border = new LinesBorder(UIManager.getColor("Table.gridColor"));
            Insets tmp = new Insets(1,1,1,1);
            Insets ins = cell.m_border.getBorderInsets(null);
            tmp.top = ins.top;
            tmp.left = ins.left;
            tmp.right = ins.right;
            tmp.bottom = ins.bottom;
            ((LinesBorder)m_border).append(tmp, true);
        }
    }
    
    /**
     * This version of the constructor constructs a formula cell.
     *
     * @param thing internal represenation of the formula
     * @param eVal the evaluated value of the formula (thing)
     * @param reference the list of cells that reference this one (can be null)
     */
    public UMCell(Formula thing, Object eVal, TreeSet reference) {
        m_expression = thing;
        m_value = eVal;
        m_refs = reference;
        m_font = null;
        m_bColor = null;
        m_fColor = null;
        m_border = null;
    }
    /**
     * This method returns the formula associated with the cell or null if it
     * does not exist.
     *
     * @return the formula (string and internal object) or null if does not
     * exist
     */
    public Formula getFormula() { return m_expression; }
    
    /** Public Method: getValue
     * Descripiton: If it is a data cell, it returns the data,
     *             of the cell. NOT USED FOR FORMULAS NOW
     * @return it returns the previously evaluated value of
     *             the formula
     */
    public Object getValue() { return m_value; }
    /** Public Method: getFont
     * Descripiton: returns a font for this cell, it can return a
     *             null if the font is not set yet.
     * @return font for the cell
     */
    public Font getFont() { return m_font; }
    /** Public Method: getBackground
     * Descripiton: returns background color for this cell, it can return a
     *             null if it is not set yet
     * @return background color
     */
    public Color getBackground() { return m_bColor; }
    
    /** Public Method: getForeground
     * Descripiton: returns foreground color for this cell, it can return a
     *             null if it is not set yet
     * @return foreground color
     */
    public Color getForeground() { return m_fColor; }
    
    /** Public Method: getBorder
     * Descripiton: returns the border for this cell
     * @return border for the cell
     */
    public Border getBorder() { return m_border; }
    
    /** Public Methods: setFont, setBackgound, setForeground
     * Descripiton: these methods set the font, background,
     *             and foreground accordingly
     * @param font sets the font
     */
    //START notice the change for sets
    public void setFont(Font font) {
        m_font = new Font(font.getAttributes());
    }
    /** This method sets the background color.
     * @param bColor the background color
     */
    public void setBackground(Color bColor) {
        m_bColor = new Color(bColor.getRGB());
    }
    /** This method sets the foreground color.
     * @param fColor the foreground color
     */
    public void setForeground(Color fColor) {
        m_fColor = new Color(fColor.getRGB());
    }
    /** This method sets the border for the cell.
     * @param border border pattern
     */
    public void setBorder(Border border) {
        if (border != null) {
            m_border = new LinesBorder(UIManager.getColor("Table.gridColor"));
            Insets tmp = new Insets(1,1,1,1);
            Insets ins = border.getBorderInsets(null);
            tmp.top = ins.top;
            tmp.left = ins.left;
            tmp.right = ins.right;
            tmp.bottom = ins.bottom;
            ((LinesBorder)m_border).append(tmp,true);
        } else
            m_border = null;
    }
    
    /** Public Method: setValue
     * Description: Sets the value field of the cell.
     *
     * @param datum he object to set the value of cell to
     */
    public void setValue(Object datum) {
        m_value = datum;
    }
    
    /** Public Method: setData
     * Description: This method changes the cell to a data cell
     *             with value datum. This method useful because
     *             changing the value does not affect the value of
     *             the reference list.
     *
     * @param datum the new value of cell
     */
    public void setData(Object datum) {
        m_value = datum;
        m_expression = null;
    }
    
    /**
     * This method sets the cell to be a formula cell. It puts the formula
     * object into the expression field. The Table of Cells is responsible
     * for recalculating and setting the appropriate value in the value
     * field of this cell.
     *
     * @param form the internal representation of formula to set this cell to
     */
    public void setFormula(Formula form) {
        m_expression = form;
    }
    /**
     * This method returns true if there are cells that reference this one.
     *
     * @return true only if there are cells that reference this one
     */
    public boolean hasRefs() {
        return (m_refs != null && !m_refs.isEmpty());
    }
    
    /**
     * This method gets the list of cells that reference this one.
     * This method should be used after the cell's value has been changed
     * to find out which cells need to be updated.
     *
     * @return all cells that reference this cell
     */
    public TreeSet getRefs() { return m_refs; }
    
    /**
     *  This method removes a cell from reference list.
     *
     * @param reference the cell to be removed from the reference list
     */
    public void removeRef(CellPoint reference) {
        if (m_refs != null) {
            m_refs.remove(reference);
            if (m_refs.isEmpty())
                m_refs = null;
        }
    }
    
    /**
     *  This method adds a cell to the dependency list.
     *
     * @param reference a new cell that references this one that needs to be
     * added to the reference list
     */
    public void addRef(CellPoint reference) {
        if (m_refs == null) { //check if refs is initiated
        m_refs = new TreeSet();
        }
        m_refs.add(reference);
    }
    
    
    /**
     * This method returns true IFF it is a formula cell
     *
     * @return true iff a formula cell
     */
    public boolean isFormula() {
        return (m_expression != null);
    }
    
    /**
     * This method determines a cell is a formula cell that has a error.
     *
     * @return true if cell is an error cell
     */
    public boolean isErrorCell() {
        return ((isFormula()) && (m_value instanceof ParserException));
    }
    
    
    /** Public Method: getType
     * Description: returns the type of data in the cell
     * @return the type of data in the cell
     */
    public int getType() {
        if (m_expression != null)
            return UMCell.FORMULA;
        if (m_value instanceof Number)
	return UMCell.FORMULA;
        return UMCell.TEXT;
    }
    
    
    /** Public Method: toString
     * Description: returns value of a cell as a string
     * @return string value of the cell
     */
		//modified by Xun
    //public String toString() {
    public String CellToString() {
        if (m_expression != null) {
            return "="+m_expression.toString();
        } else {
            return m_value.toString();
        }
        
    }
    
    /** Public Method: CellToString
     * Description: returns the string of each cell format
     * @return string format of data in cell
     */
		//modified by Xun
    //public String CellToString() {
    public String toString() {
        String temp = new String();
        //convert background color to a String
        if (m_bColor != null) {
            int blue = m_bColor.getBlue();
            temp = temp + Integer.toString(blue) + "-";
            int red = m_bColor.getRed();
            temp =temp + Integer.toString(red) + "-";
            int green = m_bColor.getGreen();
            temp = temp + Integer.toString(green) + "-/";
        }
        else {
            temp = temp + "255-255-255-/";
        }
        
        //convert foreground color to a String
        if (m_fColor != null) {
            int blue = m_fColor.getBlue();
            temp = temp + Integer.toString(blue) + "-";
            int red = m_fColor.getRed();
            temp =temp + Integer.toString(red) + "-";
            int green = m_fColor.getGreen();
            temp = temp + Integer.toString(green) + "-/";
        }
        else {
            temp = temp + "0-0-0-/";
        }
        
        //convert the font style to String
        if (m_font != null) {
            temp = temp + m_font.getName() + "-";
            temp = temp + Integer.toString(m_font.getSize()) + "-";
            temp = temp + Integer.toString(m_font.getStyle()) + "-/";
        }
        else {
            temp = temp + "Arial-12-0-/";
        }
        
        //convert border to String
        if (m_border != null) {
            Insets ins = m_border.getBorderInsets(null);
            temp = temp + Integer.toString(ins.top) + "-" + Integer.toString(ins.bottom) + "-" +
            Integer.toString(ins.left) + "-" + Integer.toString(ins.right) + "-/";
        }
        else {
            temp = temp + "0-0-0-0-/";
        }
        
        //convert either expression or value to a String
        if (m_expression != null) {
            temp = temp + "=" + m_expression.toString() + "/";
        }
        else {
            if (m_value.toString() != "")
                temp = temp + m_value.toString() + "/";
            else
                temp = temp + " /";
        }
        
        return temp;
    }
}
