import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.ListIterator;

import javax.swing.JMenu;
import javax.swing.JMenuItem;
/**
 * This class holds the history of files opened.
 * @author Spring 2004
 * @version 2.1
 */  
class FileHistory
{

	
	/** The maximum files in file history. */
		int fileMax = 5;
		/** The file list in file history. */
		LinkedList fileList = null;
		/** The file which keeps the history of the opened spreadsheets. */
		File readFile = null;
		/** The file menu. */
		JMenu fileMenu = null;
		/** The file menu operation (spreadsheet). */
		FileMenuOp m_fileMenuOp = null;
	
        /** This is the constructor for the FileHistory object which keeps the history
         * of the files opened.
         * @param spreadsheet The is the spreadsheet that is opened.
         * @param iniFile This is the iniFile which keeps the history of the opened spreadsheets.
         * @param inMenu This is what is shown in the File menu.
         */        
	public FileHistory(TerpSpreadSheet spreadsheet, File iniFile, JMenu inMenu)
	{
		fileList = new LinkedList();
		readFile = iniFile;
		fileMenu = inMenu;
		m_fileMenuOp = new FileMenuOp(spreadsheet);
		loadFileHistory();
		updateFileMenu();
		
	}

        /** This method loads the file history log.
         */        
	private void loadFileHistory()
	{
		try
	    {
			BufferedReader in = new BufferedReader(new FileReader(readFile)); 
	
			String inLine = "";
			int count = 0;
			while ((inLine = in.readLine()) != null)
			{
				count++;
				File newFile = new File(inLine);
				if (newFile.exists())
				{
					fileList.add(newFile);
				}
			}
			in.close();
		}
		catch(Exception e)
		{
			System.out.println("Exception thrown in FileHistory.loadFileHistory()");
		}	
	}
        /** This will update the file history log within the drop down
         * file menu.
         */        
	private void updateFileMenu()
	{
		// remove all JMenuItem > 8 from JMenu before going on.
		int count = fileMenu.getItemCount();
		for (int i=(count-1);i>8;i--)
		{
			fileMenu.remove(i);
		}
		fileMenu.addSeparator();
		Iterator iter = fileList.listIterator();
		count = 0;
		while(iter.hasNext())
		{
			count++;
			final File currFile = (File)iter.next();
			if (currFile.exists())
			{
				JMenuItem menuItem = new JMenuItem(count + ".  " + currFile.getName());
				menuItem.addActionListener (new ActionListener () 
				{
               		public void actionPerformed (ActionEvent evt) 
               		{
		    			m_fileMenuOp.openFile(currFile);
					}
	    		}
	    		);
				fileMenu.add(menuItem);
			}
		}
	}
	
        /** This function saves the file history to the file specified.
         * @param iniFile Saves the history to this file.
         */        
	public void saveFileHistory(File iniFile)
    {
	    //int choice = UMOptionPane.showOptionDialog(m_spreadsheet, inFileName, "Test",JOptionPane.YES_NO_CANCEL_OPTION,
	    //											JOptionPane.QUESTION_MESSAGE,null);
	    FileOutputStream out;
        PrintStream p;
        try
        {
	        File logFile = iniFile;
        	out = new FileOutputStream(logFile);
			p = new PrintStream(out);
        	ListIterator iter = fileList.listIterator();
        	while (iter.hasNext())
        	{
	        	File currFile = (File)iter.next();
	        	if (currFile.exists())
	        	{
					p.println(currFile.getPath());
				}
			}
			p.close();
			out.close();
        }
        catch (Exception e)
        {
        	System.err.println ("Error writing to logfile from FileHistory.saveFileHistory()");
        }
    }
    
        /** This saves the history and also updates the history.
         * @param inFile This is the file which the history is saved and updated to.
         */        
    public void addFileToHistoryAndUpdate(File inFile)
    {
	    int count = fileList.size();
	    
	    if (!fileList.contains(inFile))
	    {
	if ((count + 1) <= fileMax)
	    	{
		    	fileList.removeLast();
	    	}
	    	fileList.addFirst(inFile);
    	}
    	else
    	{
	    	fileList.remove(inFile);
	    	fileList.addFirst(inFile);
    	}
    	updateFileMenu();
    }
}
