import java.awt.Button;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.image.BufferedImage;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.StringTokenizer;

import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.table.TableColumn;

import com.activetree.print.AtTablePrinter;
import com.activetree.print.PageNoPainter;

/** This class will implement all of the actions for the 'File' menu.
 * @author Spring 2004
 * @version 2.1
 */
public class FileMenuOp implements ActionListener, Printable {
   
    
	/** Dialog */
	   JDialog m_dialog;
	   /** Ok button */
	   Button ok;
	   /**
		* Cancel button
		*/
	   Button cancel;
	   /** UNNAMED value*/
	   final static private int UNNAMED = 0;
	   /** FILE value*/
	   final static private int FILE = 1;
	   /** DATABASE value*/
	   final static private int DATABASE = 2;
    
    //private int openType = UNNAMED;
	/** maxNumPage value*/
	private int maxNumPage = 1;
	/** columnOverflow flag*/
	private boolean columnOverflow;
	/** closeCancel value*/
	private int closeCancel = 0;
	

    
    //private StringTokenizer tokenizer;
    
	/** The SpreadSheet */
    private TerpSpreadSheet m_spreadsheet;
    
	/** The JTable */
		private JTable m_table;
		/** The TableModel */
		private UMTableModel m_tableModel;
		/** The File Chooser */
		private JFileChooser m_fileChooser;
		/** The File */
		private File m_file;
		/** The extension value */
		final private static String m_extname = ".ums";
		/** The FileFilter */
		private static UMFileFilter m_filter;
		/** The Print Stream */
		private PrintStream m_out;
		/** The Buffered reader */
		private BufferedReader m_in;
		/** The Buffered Image */
		private BufferedImage m_img;
		/** The PrintOption value */
    private int m_printOption = 0;
    
	/** UMFileFilter class takes care of filtering files for the correct extension
		 * 
		 * @author Spring 2004
		 * @version 2.1
		 */
    class UMFileFilter extends javax.swing.filechooser.FileFilter {
        /** If a file should be accepted.
         * @param m_file The file to be accpted.
         * @return true if File is correct, false otherwise.
         */
        public boolean accept(File m_file) {
            if (m_file.isDirectory()) {
                return true;
            }
            
            return m_file.getName().endsWith(m_extname);
        }
        
        /** Returns the discription of file.
         * @return The discription as a string.
         */
        public String getDescription() {
            return "TerpSpreadSheet (*.ums)";
        }
    }
    /** Public Constructor: FileMenuOp
     * Description: This constructs the FileMenuOp and set the parent
     *                   application
     *
     * @param spreadsheet The TerpSpreadSheet application
     */
    public FileMenuOp(TerpSpreadSheet spreadsheet) {
        m_spreadsheet = spreadsheet;
        m_table = spreadsheet.getTable();
        m_tableModel = spreadsheet.getTableModel();
        m_fileChooser = new JFileChooser(new File("."));
        m_filter = new UMFileFilter();
        m_fileChooser.addChoosableFileFilter(m_filter);
        
        //----------------------------
        // following was added by JJdESi4
        m_fileChooser.setAccessory(new FilePreview(m_fileChooser));
        //----------------------------
        
        m_table.registerKeyboardAction(this,"New", KeyStroke.getKeyStroke
        (KeyEvent.VK_N,ActionEvent.CTRL_MASK,false),  JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"OpenFile", KeyStroke.getKeyStroke
        (KeyEvent.VK_O,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"Save", KeyStroke.getKeyStroke
        (KeyEvent.VK_S,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"Print", KeyStroke.getKeyStroke
        (KeyEvent.VK_P,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
        
        m_table.registerKeyboardAction(this,"Exit", KeyStroke.getKeyStroke
        (KeyEvent.VK_E,ActionEvent.CTRL_MASK,false), JComponent.WHEN_FOCUSED);
    }
    
    /** Confirm dialog box that queries whether user wishes to save current document or not.
     * @return true to save, false otherwise
     */
    public int queryForSave() {
        // show confirm dialog box from static JOptionPane method
        String filename = "Untitled";
        if (m_file != null)
            filename = m_file.getName();
        
        int choice =
        UMOptionPane.showOptionDialog
        (m_spreadsheet,
        "Do you want to save the changes you made to \""+filename+"\"?",
        "Save",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null);
        // return user's choice
        return choice;
    }
    
    /** Confirm dialog box that queries whether user wishes to close current document or not.
     *
     * @return true to close, false otherwise
     */
    public int queryForClose() {
        // show confirm dialog box from static JOptionPane method
        int choice =
        UMOptionPane.showOptionDialog
        (m_spreadsheet,
        "Do you want to close the current spreadsheet?",
        "close",
        JOptionPane.YES_NO_CANCEL_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null);
        // return user's choice
        return choice;
    }
    
    /** This will perform new file operation.
     */
    public void newFile() {
        if (m_tableModel.isModified()) {
            //if the table has been modify then ask if the user would like to save
            int choice = queryForSave();
            if (choice == 0) {
                saveFile();
            }
            else if (choice == 2) {
                //if user has cancel the new command it doesn't do any but return the function
                return;
            }
        }
        /*
        //create a new table
        TerpSpreadSheet newSpreadSheet = new TerpSpreadSheet();
        //get the table model from the new table
        UMTableModel newTable = newSpreadSheet.getTableModel();
        //convert each empty cell to a string to text
        String text = newTable.ConvertToString();
        m_tableModel = m_spreadsheet.getTableModel();
        //reset the table from the string, text, it should set every
        //cell to an empty cell
        System.out.println( TerpSpreadSheet.m_maxRow ) ;
        m_tableModel.CellfromString(text, 0, 0,
        new CellRange(TerpSpreadSheet.m_baseRow,
        TerpSpreadSheet.m_maxRow,
        TerpSpreadSheet.m_baseCol,
        50));
         */
        m_tableModel.clearRange( new CellRange( TerpSpreadSheet.m_baseRow, TerpSpreadSheet.m_maxRow, TerpSpreadSheet.m_baseCol, TerpSpreadSheet.m_maxCol )) ;
        
        CellRange cells = new CellRange( TerpSpreadSheet.m_baseRow, TerpSpreadSheet.m_maxRow, TerpSpreadSheet.m_baseCol, TerpSpreadSheet.m_maxCol ) ;
        Color color = Color.WHITE ;
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_tableModel.getCellAt(i,j);
                cell.setBackground(color);
            }
        color = Color.BLACK ;
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_tableModel.getCellAt(i,j);
                cell.setForeground(color);
            }
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_tableModel.getCellAt(i,j);
                cell.setBorder(null);
            }
        for (int j = 1; j <= TerpSpreadSheet.m_maxCol; j++) {
            TableColumn column = m_spreadsheet.getTable().getColumnModel().getColumn(j);
            column.setMinWidth(80);
            column.setPreferredWidth(80);
        }
        m_table.setRowHeight( 16 ) ;
        
        //reset the modification flag back to false
        m_tableModel.setModified(false);
        //reset the title and the filename
        m_spreadsheet.setTitle("TerpSpreadsheet");
        m_file = null;
    }
    
    /** This will perform open file operation.
     */
    public void openFile() {
        /*UMOptionPane.showMessageDialog(m_spreadsheet,
                                       "This is the open file operation",
                                       "TerpSpreadSheet Info",
                                       JOptionPane.WARNING_MESSAGE,
                                       null);*/
        
        int choice = m_fileChooser.showOpenDialog(m_spreadsheet);
        System.out.println("I have choice " + choice);
        if(choice == JFileChooser.APPROVE_OPTION) {
            File selectedfile = m_fileChooser.getSelectedFile();
            m_fileChooser = new JFileChooser(selectedfile.getParentFile());
            m_fileChooser.addChoosableFileFilter(m_filter);
            
            
            //----------------------------------------
            // following was added by JJdESi4
            System.out.println("openfile()");
            m_fileChooser.setAccessory(new FilePreview(m_fileChooser, selectedfile));
            //----------------------------------------
            
            openTableModel(selectedfile);
            //System.out.println("I'll open a file");
            
            //Newly added - JJ
            (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(selectedfile);
            
        }
        else {
            //System.out.println("I won't open a file");
        }
    }
    
    //Newly added - JJ
    /** This will perform open file operation specified.
     * @param inFile The file name to be opened.
     */
    public void openFile(File inFile) {
        File selectedfile = inFile;
        m_fileChooser = new JFileChooser(selectedfile.getParentFile());
        m_fileChooser.addChoosableFileFilter(m_filter);
        
        //---------------------------
        // following was added by JJdESi4
        System.out.println("openFile(File inFile)");
        m_fileChooser.setAccessory(new FilePreview(m_fileChooser, selectedfile));
        //---------------------------
        
        openTableModel(selectedfile);
        
        (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(selectedfile);
        
    }
    
    /** Opens table model data from file.
     * @param aFile file name from which to open
     */
    public void openTableModel(File aFile) {
        
        StringBuffer textBuf = new StringBuffer();
        String line;
        //String password = null;
        int numRowsTemp = 0 ;
        int numColsTemp = 0 ;
        try {
            m_in =  new BufferedReader(new FileReader(aFile));
            while ((line = m_in.readLine()) != null) {
                textBuf.append(line);
                StringTokenizer a = new StringTokenizer( line, "\t" ) ;
                numColsTemp = a.countTokens() - 1;
                textBuf.append("\n");
                numRowsTemp++ ;
            }
            
            numRowsTemp-- ;
            
            String text = textBuf.toString();
            m_tableModel = m_spreadsheet.getTableModel();
            int count = numRowsTemp ;
            int max = TerpSpreadSheet.m_maxRow ;
            if ( count > max )
                while ( count - max > 0 ) {
                    m_tableModel.insertRow( m_tableModel.getSelection() ) ;
                    TerpSpreadSheet.m_maxRow++ ;
                    count-- ;
                }
            count = numColsTemp ;
            max = TerpSpreadSheet.m_maxCol ;
            if ( count > max )
                while ( count - max > 0 ) {
                    m_tableModel.insertColumn( m_tableModel.getSelection() ) ;
                    TerpSpreadSheet.m_maxCol++ ;
                    count-- ;
                }
            m_tableModel.CellfromString(text, 0, 0,
            new CellRange(TerpSpreadSheet.m_baseRow,
            numRowsTemp,
            TerpSpreadSheet.m_baseCol,
            numColsTemp));
            /*m_tableModel.fromString1(text, 0, 0,
                                  new CellRange(TerpSpreadSheet.m_baseRow,
                                                50,
                                                TerpSpreadSheet.m_baseCol,
                                                50));*/
            
            m_file = aFile;
            
            String title = m_file.getName();
            if (title.endsWith(m_extname))
                title = title.substring(0, title.length()-4);
            
            m_spreadsheet.setTitle(title+" - TerpSpreadSheet");
            
        } catch (FileNotFoundException e) {
            fileOpError("Open", "File \""+aFile.getName()+"\" not found!");
        } catch (IOException e) {
            fileOpError("Open", "I/O error in opening \""+aFile.getName()+"\"!");
        } catch (Exception e) {
        }
    }
    
    /** This will perform save file operation.
     */
    public void saveFile() {
        /*UMOptionPane.showMessageDialog(m_spreadsheet,
                                       "This is the save file operation",
                                       "TerpSpreadSheet Info",
                                       JOptionPane.WARNING_MESSAGE,
                                       null);*/
        if(m_tableModel.isModified()) {
            if (m_file == null) {
                saveAsFile();
            }
            else
            {System.out.println("not null");
             //Just save without asking
             m_tableModel.setModified(false);
             saveTableModel();
             //Newly added - JJ
             (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(m_file);
             
            }
        }
        //SaveDialogWindow();
    }
    
    
    /** This will perform saveAs file operation.
     */
    public void saveAsFile() {
        //display the Save As dialog box
        int choice = m_fileChooser.showSaveDialog(m_spreadsheet);
        if(choice == JFileChooser.APPROVE_OPTION) {
            // get selected file to save to
            File selectedfile = m_fileChooser.getSelectedFile();
            
            m_fileChooser = new JFileChooser(selectedfile.getParentFile());
            m_fileChooser.addChoosableFileFilter(m_filter);
            
            
            //---------------------------
            // following was added by JJdESi4
            System.out.println("SaveAsFile");
            m_fileChooser.setAccessory(new FilePreview(m_fileChooser, selectedfile));
            //---------------------------
            
            // enforce the extension name
            String path = selectedfile.getParent() ;
            String file = selectedfile.getName() ;
            
            if ( !(file.startsWith( "\"" ) && file.endsWith( "\"" )) ) {
                System.out.println( "No quotes" );
                if (!file.endsWith(m_extname))
                    selectedfile = new File( path + "\\" + file + m_extname);
                else
                    selectedfile = new File( path + "\\" + file );
            }
            else {
                System.out.println( "Quotes" ) ;
                file = file.substring( 1, file.length() - 1 );
                selectedfile = new File( path + "\\" + file ) ;
            }
            // check the existence of the file
            
            
            
            if (selectedfile.exists()) {
                choice =
                UMOptionPane.showOptionDialog
                (m_spreadsheet,
                "File "+selectedfile.getName()+" already exists.\n\n"+
                "Do you want to overwrite it?\n\n",
                "Save",
                JOptionPane.YES_NO_OPTION,
                JOptionPane.WARNING_MESSAGE,
                null, 1);
                if (choice != JOptionPane.YES_OPTION) {   //if user choose not to overwrite the file then close
                    closeCancel = 1;
                    return;
                }
            }
            //save the table
            m_tableModel.setModified(false);
            saveTableModel(selectedfile);
            //Newly added - JJ
            (m_spreadsheet.getFileHistory()).addFileToHistoryAndUpdate(selectedfile);
            
            
        }
        else{
            closeCancel = 1;
        }
    }
    
    
    /** Saves table model to specified file cell by cell in tab-delimited format.
     *
     * @param aFile the file name to save to.
     */
    public void saveTableModel(File aFile) {
        
        try {
            
            // initialize output
            m_out = new PrintStream(new FileOutputStream(aFile));
            
            int rowCount = m_tableModel.getRowCount();
            int colCount = m_tableModel.getColumnCount();
            // print out number of columns and rows
            //	    out.println(rowCount+"\t"+colCount);
            
            m_out.print(m_tableModel.ConvertToString());
            m_out.flush();
            
            m_file = aFile;
            // set modified to false
	m_tableModel.setModified(true);
            
            String title = m_file.getName();
            if (title.endsWith(m_extname))/*FAULT:	if (!(title.endsWith(m_extname)))*/
                title = title.substring(0, title.length()-4);
            m_spreadsheet.setTitle(title+" - TerpSpreadSheet");
        } catch (FileNotFoundException e) {
            fileOpError("Save", "File \""+aFile.getName()+"\" cannot be created!");
        } catch (IOException e) {
            fileOpError("Save", "I/O error in saving \""+aFile.getName()+"\"!");
        }
    }
    
    
    /** Overloaded save function that takes the current filename  as default.
     */
    public void saveTableModel() {
        saveTableModel(m_file);
    }
    
    /** Use for printing the file that is being sent from terp manager.
     * @param file The file which is to be printed.
     */
    public void setFile( File file ) { this.m_file = file; }
    
    /** This will perform print file operation.
     */
    public void printFile() {
        m_printOption =1;
        Object[] options = {"Print Selected Area",
        "Print Entire Docuement", "Preview",
        "Cancel"};
        int n = JOptionPane.showOptionDialog(m_spreadsheet,
        "Select what you wish to do",
        "Print Option",
        JOptionPane.DEFAULT_OPTION,
        JOptionPane.QUESTION_MESSAGE,
        null,options,options[3]);
        AtTablePrinter printer = new AtTablePrinter();
        if (n == 3 || n == -1)
            return;
        m_printOption = n;
        if ( n == 2 ) {
            printer.preview( m_table, PageNoPainter.LOWER_CENTER, false ) ;
            return ;
        }
        else if ( n == 1 ) {
            printer.setDrawCellBorder(true);
            printer.setRepeatHeader(true);
            printer.setHeaderBold(false);
            printer.print(m_table, PageNoPainter.LOWER_CENTER);
        }
        else if ( n == 0 ) {
            printData() ;
        }
    }
	/** This will perform print file operation.
		*/
    public void printData() {
        
        // create temporary object that contains this FileOp object
        // and reset columnOverflow to false
        FileMenuOp fileOp = this;
        columnOverflow = false;
        
        try {
            PrinterJob prnJob = PrinterJob.getPrinterJob();
            prnJob.setPrintable(fileOp);
            
            // opens print dialog, and if user doesn't cancel the dialog
            // it calls print()
            prnJob.print();
        }
        catch (PrinterException e) {
            e.printStackTrace();
            fileOpError("Print", "Printing error: "+e.toString());
        }
    }
    
    
    /** Main print method of FileOp that performs calculations for
     * dimensions of printed table, and writes the strings out to
     * the <code>Graphics</code> object that is responsible for
     * holding the printed data.  All parameters are called by Java
     * classes, specifically <code>PrintJob</code>, not explicitly in
     * our code
     *
     * @param pg Graphics object that holds the printed information
     * @param pageFormat PageFormat object that holds print dimensions
     * @param pageIndex number of current Page
     * @return status of current call of print method
     * @throws PrinterException Thrown if cannot print.
     */
    public int print(Graphics pg, PageFormat pageFormat,
    int pageIndex) throws PrinterException {
        
        
        // if pageIndex is beyond maximum page number, then
        // don't return
        // set Graphics object to printed cartesian origin
        pg.translate((int)pageFormat.getImageableX(),
        (int)pageFormat.getImageableY());
        
        // get printable width and height of page
        int wPage = 0;
        int hPage = 0;
        
        
        wPage = (int)pageFormat.getImageableWidth();
        hPage = (int)pageFormat.getImageableHeight();
        
        
        
        if ((pageIndex == 0) && (m_printOption == 0) ) {
            m_img = m_tableModel.getSelection().getImage(true);
        }
        int width = m_img.getWidth();
        int height = m_img.getHeight();
        int currH = 0;
        int currW = 0;
        BufferedImage img;
        for (int i = 1; i <= pageIndex; i++) {
            currW = Math.min(currW + wPage, width);
            if (currW == width) {
                currW = 0;
                currH = Math.min(currH + hPage, height);
            }
            if (currH == height)
                return NO_SUCH_PAGE;
            
        }
        
        img = m_img.getSubimage(currW, currH,
        Math.min(wPage, width - currW ),
        Math.min(hPage, height - currH));
        pg.drawImage(img, 0, 0, null);
        
        // explicit call on garbage collector to flush all temporary data
        // used, such as graphics objects
        System.gc();
        return PAGE_EXISTS;
    }
    
    
    /** This will perform exit file operation.
     * @param operation which exit operation to perform.
     */
    public void exitFile(int operation) {
        //reset the flag to close the file
        closeCancel = 0;
        //check if the table has been modified or not, if has then
        //ask if the user wants to save before closing the table
        if (m_tableModel.isModified()) {/*FAULT:	if (!(m_tableModel.isModified())) {*/
            int choice = queryForSave();
            if (choice == 0) {/*FAULT:	if (choice != 0) {*/
                saveFile();
            }
            else if ((choice == 2) || ( choice == -1 ) ) {/*FAULT:	else if ((choice != 2) && ( choice != -1 ) ) {*/
                return;
            }
        }
        //Exit isn't from the default operation
        if (closeCancel == 0) {
            m_spreadsheet.dispose();
            if (m_spreadsheet.m_isStandAlone)
                System.exit(0);
        }
    }
    
    
    /** This method is activated on the Keystrokes we are listening to
     * in this implementation. Here it listens for keystroke
     * ActionCommands.
     *
     * Without this listener, when we press certain keys the individual
     * cell will be activated into editing mode in addition to the
     * effect of the key accelerators we defined with menu items.
     * With this key listener, we avoid this side effect.
     * @param e The actioncommand to be completed.
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().compareTo("New") == 0) {
            newFile();
        }
        else if (e.getActionCommand().compareTo("OpenFile") == 0) { // CHANGED "Open" -> "OpenFile"
            openFile();
        }
        else if (e.getActionCommand().compareTo("Save") == 0) {
            saveFile();
        }
        else if (e.getActionCommand().compareTo("Print") == 0) {
            printFile();
        }
        else if (e.getActionCommand().compareTo("Exit") == 0) {
            m_spreadsheet.exit();
        }
    }
    /**
     * a function to display error messages
     * @param op the operation that caused this error
     * @param error the error message
     */
    private void fileOpError(String op, String error) {
        UMOptionPane.showMessageDialog(m_spreadsheet, error, op,
        JOptionPane.ERROR_MESSAGE, null);
        m_table.requestFocus();
    }
    
    /** Returns the file.
     * @return The File object.
     */
    public File getFile() {
        return m_file;
    }
}
