//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//CMSC435  FormatMenuOp (some of the code is adapted from
//                        SharpTools project )
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.StringTokenizer;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.SwingConstants;
import javax.swing.table.TableColumn;

/**
 * Class: EditMenuOp
 * Description:  This class will implement all the functions for the
 *               format menu
 * Disclaimer: Some of the code and and ideas we found online in
 *            the ShartTools project.  We thank them for making
 *            their code availible.
 * @author TSS
 * @version  3.0
 */
public class FormatMenuOp {

    
	/** The spreadsheet */
	TerpSpreadSheet m_spreadsheet;
		/**The Table Model*/
		UMTableModel m_model;
		/** Format Cells Pane*/
		FormatCellsTabs m_pane;
		/** Dialog */
		JDialog m_dialog;
		/** JTable */
		JTable table;
		/** M_T value */
		int m_t = 2;
    
    /**
     * Public Constructor: FormatMenuOp
     * Description: This constructs the FormatMenuOp and set the parent
     *              application
     *
     * @param spreadsheet the TerpSpreadSheet application
     */
    public FormatMenuOp(TerpSpreadSheet spreadsheet) {
        m_spreadsheet = spreadsheet;
        m_model = spreadsheet.getTableModel();
        table = m_spreadsheet.getTable();
        
        // costruct the format cells dialog
        m_dialog = new JDialog(m_spreadsheet, "Format Cells", true);
        m_pane = new FormatCellsTabs(this, m_dialog);
        m_dialog.addWindowListener(new WindowAdapter() {
            
            public void windowOpened(WindowEvent we) {
            }
            
            public void windowClosing(WindowEvent we) {
                m_dialog.setVisible(false);
            }
        });
        m_dialog.setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        m_dialog.getContentPane().setLayout(new BorderLayout());
        
        Point p = m_spreadsheet.getLocation();
        Point p1 = new Point((int) (p.getX() + 50),
        (int) (p.getY() + 50));
        
        m_dialog.setLocation(p1);
        m_dialog.getContentPane().add(m_pane, BorderLayout.CENTER);
        m_dialog.pack();
        
        // Identifying the Insert/Delete KeyStroke user can modify this
        // to some other Key combination.
        
        KeyStroke insertrow = KeyStroke.getKeyStroke
        (KeyEvent.VK_INSERT,
        ActionEvent.CTRL_MASK,
        false);
        
        KeyStroke insertcolumn = KeyStroke.getKeyStroke
        (KeyEvent.VK_INSERT,
        ActionEvent.CTRL_MASK|ActionEvent.SHIFT_MASK,
        false);
        
        KeyStroke deleterow = KeyStroke.getKeyStroke
        (KeyEvent.VK_DELETE,
        ActionEvent.CTRL_MASK,
        false);
        
        KeyStroke deletecolumn = KeyStroke.getKeyStroke
        (KeyEvent.VK_DELETE,
        ActionEvent.CTRL_MASK|ActionEvent.SHIFT_MASK,
        false);
    }
    
    
    /** This will perform the columnWidth operation
     */
    public void columnWidthOp() {
        Integer initValue = getColumnWidth(getSelected());
        boolean done = false;
        Integer result = null;
        do {
            result = UMOptionPane.showIntegerInputDialog(m_spreadsheet, "Enter column width",
            "Column Width", JOptionPane.QUESTION_MESSAGE, null, initValue);
            if (result == null)
                done = true;
            else if (result.intValue() < 1 || result.intValue() > 250)
                UMOptionPane.showMessageDialog( m_spreadsheet, "Enter an integer value between 1 and 250",
                "Invalid Column width", JOptionPane.ERROR_MESSAGE, null);
            else
                done = true;
        } while (!done);
        
        if (result != null) {
            setColumnWidth(result.intValue(), getSelected());
            m_spreadsheet.getTable().updateUI();
            m_model.setModified(true);
        }
    }
    
    /** This will perform row height operation
     */
    public void rowHeightOp() {
        Integer initValue = getRowHeight(getSelected());
        boolean done = false;
        Integer result = null;
        do {
            result = UMOptionPane.showIntegerInputDialog(m_spreadsheet, "Enter row height",
            "Row Height", JOptionPane.QUESTION_MESSAGE, null, initValue);
            if (result == null)
                done = true;
            else if (result.intValue() < 1 || result.intValue() > 250)
                UMOptionPane.showMessageDialog( m_spreadsheet, "Enter an integer value between 1 and 250",
                "Invalid Row Height", JOptionPane.ERROR_MESSAGE, null);
            else
                done = true;
        } while (!done);
        
        if (result != null) {
            setRowHeight(result.intValue(), getSelected());
            m_spreadsheet.getTable().updateUI();
            m_model.setModified(true);
        }
    }
    
    /** Auto fits the columns
     */
    public void columnAutofitOp() {
        int maxWidth = -1;
        int width = -1;
        String s = m_model.toString(getSelected(), true);
        StringTokenizer stkn = new StringTokenizer(s);
        int toks = stkn.countTokens();
        for (int i = 0; i < toks; i++) {
            FontMetrics metrics = m_model.getTable().getGraphics().getFontMetrics();
            width = metrics.stringWidth(stkn.nextToken()) + (2*m_model.getTable().getColumnModel().getColumnMargin());
            if (width > maxWidth)
                maxWidth = width;
        }
        if (maxWidth > 0) {
            setColumnWidth(maxWidth+2, m_model.getSelection());
            m_spreadsheet.getTable().updateUI();
            m_model.setModified(true);
        }
    }
    
    /** This will perform the format cells operation
     */
    public void formatCellsOp() {
        CellRange cells = getSelected();
        // get intial states, and display the format cells dialog
        m_pane.displayDialog(isBold(cells),
        isItalic(cells),
        isRegular(cells),
        getFontSize(cells),
        getFontName(cells),
        getBackground(cells),
        getForeground(cells),
        isInsideBorder(cells),
        isOutsideBorder(cells),
        isNoBorder(cells));
        m_dialog.show();
        
    }
    
    
    /** This is called by the format cells dialog, to signal what kind of border needs to be drawn
     *
     * @param inside true if inside.
     * @param outside true if outside
     * @param none true if none, false otherwise.
     */
    public void doAllBorder(boolean inside,
    boolean outside,
    boolean none) {
        CellRange cells = getSelected();
        if (none)
            setNoBorder(cells);
        
        if (inside && outside) {
            setInsideBorder(true, cells);
            setOutsideBorder(false,cells);
        }
	else if (!inside)
            setInsideBorder(true,cells);/*FAULT:	setInsideBorder(false,cells);*/
        else if (outside)/*FAULT:	else if (!outside)*/
            setOutsideBorder(true,cells);/*FAULT:	setOutsideBorder(false,cells);*/
        
        
        m_spreadsheet.getTable().updateUI();
        m_model.setModified(true);/*FAULT:	m_model.setModified(false);*/
    }
    
    /** Called when the dialog need to be displayed, sets everything to correct intial values
     * @param bold true if bold
     * @param italic true if italic
     * @param regular true if regular font.
     * @param fontSize the font size.
     * @param fontName The font name.
     * @param bColor The font color.
     * @param fColor The font color.
     */
    public void doAllFont(boolean bold,
    boolean italic,
    boolean regular,
    String fontName,
    String fontSize,
    Color bColor,
    Color fColor) {
        CellRange cells = getSelected();
        Component cmp = getEditor();
        if (bold)/*FAULT:	if (!bold)*/
            setBold(true, cells, cmp);/*FAULT:	setBold(false, cells, cmp);*/
        if (italic)/*FAULT:	if (!italic)*/
            setItalic(true, cells, cmp);/*FAULT:	setItalic(false, cells, cmp);*/
        if (regular)/*FAULT:	if (!regular)*/
            setRegular(cells, cmp);
        if (fontName.compareTo("[Automatic]") != 0)/*FAULT:	if (fontName.compareTo("[Automatic]") == 0)*/
            setFontStyle(fontName, getSelected(), getEditor());
        if (fontSize.compareTo("-") != 0) {/*FAULT:	if (fontSize.compareTo("-") == 0) {*/
            try {
                float fsize = Float.parseFloat(fontSize);
                setFontSize(fsize, cells, cmp);
            }
            catch (Exception e) {
                System.out.println("Error in parsing size No action taken");
            }
        }
        if (bColor != null)/*FAULT:	if (bColor == null)*/
            setBackgroundColor(bColor, cells, cmp);
        if (fColor != null)/*FAULT:	if (fColor == null)*/
            setForegroundColor(fColor, cells, cmp);
        
        m_spreadsheet.getTable().updateUI();
        m_model.setModified(true);/*FAULT:	m_model.setModified(false);*/
    }
    
    //***************************************************************
    // Public Method: setBoldOp
    // Description: This is the bold operation (shortcut button)
    //**************************************************************
    /** This is the bold operation (shortcut button)
     * @param set if bold, false otherwise.
     */
    public void setBoldOp(boolean set) {
        setBold(set, getSelected(), getEditor());
        m_spreadsheet.getTable().updateUI();
        m_model.setModified(true);/*FAULT:	m_model.setModified(false);*/
    }
    
    /** This is the set italic operation (shortcut button)
     * @param set true if italic is set.
     */
    public void setItalicOp(boolean set) {
        setItalic(set, getSelected(), getEditor());
        m_spreadsheet.getTable().updateUI();
        m_model.setModified(true);/*FAULT:	m_model.setModified(false);*/
    }
    
    /** This is the set font operation (shrotcut)
     * @param font true if font is set
     */
    public void setFontOp(Object font) {
        if (((String) font).compareTo("[Automatic]") == 0)/*FAULT:	if (((String) font).compareTo("[Automatic]") != 0)*/
            return;
        setFontStyle((String) font, getSelected(), getEditor());
        
        
        m_spreadsheet.getTable().updateUI();
        m_model.setModified(true);/*FAULT:	m_model.setModified(false);*/
    }
    
    
    /** This is the set font size operation (shortcut)
     * @param size set the size of font
     */
    public void setFontSizeOp(Object size) {
        if (((String) size).compareTo("-") == 0)/*FAULT:	if (((String) size).compareTo("-") != 0)*/
            return;
        try {
            float fsize = Float.parseFloat((String) size);
            setFontSize(fsize, getSelected(), getEditor());
            m_spreadsheet.getTable().updateUI();
            m_model.setModified(true);/*FAULT:	m_model.setModified(false);*/
        }
        catch (Exception e) {
            System.out.println("Error in parsing size No action taken");
        }
    }
    
    //**************************************************************
    //Private Method: getSelected
    //Description: return a list of selected cells, only one cell
    //             if it is being edited
    //**************************************************************
    /** return a list of selected cells, only one cell if it is being edited
     * @return list of selected cells
     */
    public CellRange getSelected() {
        if (m_spreadsheet.getTable().isEditing())
            return new CellRange(m_spreadsheet.getTable().getEditingRow(),
            m_spreadsheet.getTable().getEditingRow(),
            m_spreadsheet.getTable().getEditingColumn(),
            m_spreadsheet.getTable().getEditingColumn());
        
        return m_model.getSelection();
    }
    
    //*******************************************************************
    //Private Method: getEditor
    //Description: returns the current editing component or null
    //*******************************************************************
    /** returns the current editing component or null
     * @return current editing component or null
     */
    public Component getEditor() {
        Component editor = m_spreadsheet.getTable().getEditorComponent();
        if (editor != null) {
            JTextField tField = (JTextField) editor;
            int row = m_spreadsheet.getTable().getEditingRow();
            int col = m_spreadsheet.getTable().getEditingColumn();
            m_model.setValueAt(tField.getText(), row, col);
        }
        return editor;
    }
    
    //*******************************************************************
    //Private Method: setFontStyle
    //Description: resets the font to the new style
    //*******************************************************************
	/**
		* Private Method: setFontStyle 
		* Description: resets the font to the new style
		* @param editor the Editor
		* @param cells The cell range
		* @param style The Style string
		*/
    private void setFontStyle(String style,
    CellRange cells,
    Component editor) {
        // Set style for the selection
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                Font font = cell.getFont();
                if (font == null)
                    font = m_spreadsheet.getTable().getFont();
                float size = font.getSize2D();
                font = new Font(style, font.getStyle(), font.getSize());
                font.deriveFont(size);
                cell.setFont(font);
                
            }
        
        // Set style for the editor
        if (editor != null) {
            Font font = editor.getFont();
            float size = font.getSize2D();
            font = new Font(style, font.getStyle(), font.getSize());
            font.deriveFont(size);
            editor.setFont(font);
        }
        
        // Update SpreadSheet gue
        for (int i = 0; i < m_spreadsheet.m_fontList.getItemCount(); i++) {
            if (((String)m_spreadsheet.m_fontList.getItemAt(i)
            ).compareTo(style) == 0) {
                m_spreadsheet.m_fontList.setSelectedIndex(i);
                break;
            }
        }
    }
    
    //*******************************************************************
    //Private Method: setFontSize
    //Description: resets the font to the new size, also resets the
    //             height of the columns if they are small
    //*******************************************************************
	/**
	 *Private Method: setFontSize
	   Description: resets the font to the new size, also resets the
				height of the columns if they are small
	 * @param editor the Editor
	 * @param cells The cell range
	 * @param size The float size
	 */
    private void setFontSize(float size,
    CellRange cells,
    Component editor) {
        // Set font size for selection
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++) {
            if (size > m_spreadsheet.getTable().getRowHeight(i))
                setRowHeight((int) size + 1,cells);
            else if ( size < m_spreadsheet.getTable().getRowHeight(i))
                setRowHeight(16, cells) ;
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                Font font = cell.getFont();
                if (font != null)
                    cell.setFont(font.deriveFont(size));
                else {
                    font = m_spreadsheet.getTable().getFont();
                    cell.setFont(font.deriveFont(size));
                }
            }
        }
        
        // Set font size for the editor
        if (editor != null) {
            Font font = editor.getFont();
            font = font.deriveFont( size);
            editor.setFont(font);
        }
        
        // Update the spreadsheet gue
        Integer intSize = new Integer((int) size);
        for (int i = 0; i < m_spreadsheet.m_fontSizeList.getItemCount(); i++) {
            if (((String)m_spreadsheet.m_fontSizeList.getItemAt(i)
            ).compareTo(intSize.toString()) == 0) {
                m_spreadsheet.m_fontSizeList.setSelectedIndex(i);
                break;
            }
        }
    }
    //*******************************************************************
    //Private Method: setBackgroundColor
    //Description: sets the new background color
    //*******************************************************************
	/**
		 *Private Method:  setBackgroundColor
		  Description: sets the new background color
		 * @param editor the Editor
		 * @param cells The cell range
		 * @param color The Color		 */
    private void setBackgroundColor(Color color,
    CellRange cells,
    Component editor) {
        // Set background color for the selection
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                cell.setBackground(color);
            }
        
        // set background color for the editor
        if (editor != null) {/*FAULT:	if (editor == null) {*/
            editor.setBackground(color);
        }
    }
    //*******************************************************************
    //Private Method: setForegroundColor
    //Description: sets the new foreground color
    //*******************************************************************
	/**
				 *Private Method:  setForegroundColor
				 Description: sets the new foreground color
				 * @param editor the Editor
				 * @param cells The cell range
				 * @param color The Color		 */
    private void setForegroundColor(Color color,
    CellRange cells,
    Component editor) {
        // set forground color fot the selection
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                cell.setForeground(color);
            }
        // set forground color for the editor
        if (editor != null) {/*FAULT:	if (editor == null) {*/
            editor.setForeground(color);
        }
    }
    
    //*******************************************************************
    //Private Method: setBold
    //Description: either sets of unsets the font to be bold
    //*******************************************************************
	/**
					 *Private Method:  setBold
					 Description: either sets of unsets the font to be bold
					 * @param editor the Editor
					 * @param cells The cell range
					 * @param set boolean		 */
    private void setBold(boolean set,
    CellRange cells,
    Component editor) {
        // set all selected cells to bold
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);/*FAULT:	UMCell cell = m_model.getCellAt(j,i);*/
                Font font = cell.getFont();
                boolean italic = font.isItalic();
                font = font.deriveFont(Font.PLAIN);
                int st = 0;
                // keep previous setting as well
                if (italic)/*FAULT:	if (!italic)*/
                    st = Font.ITALIC;
                if (set)/*FAULT:	if (!set)*/
                    st += Font.BOLD;
                font = font.deriveFont(st);
                cell.setFont(font);
            }
        
        // set editor font to bold
        if (editor != null) {/*FAULT:	if (editor == null) {*/
            Font font = editor.getFont();
            boolean italic = font.isItalic();
            font = font.deriveFont(Font.PLAIN);
            int st = 0;
            if (italic)
                st = Font.ITALIC;
            if (set)
                st += Font.BOLD;
            font = font.deriveFont(st);
            editor.setFont(font);
        }
        // update spreadsheet gue
        m_spreadsheet.boldButton.setSelected(true);/*FAULT:	m_spreadsheet.boldButton.setSelected(false);*/
    }
    
    //*******************************************************************
    //Private Method: setItalic
    //Description: either sets or unsets font to be italic
    //*******************************************************************
	/**
						 *Private Method:  setItalic
						 Description: either sets or unsets font to be italic
						 * @param editor the Editor
						 * @param cells The cell range
						 * @param set boolean		 */
    private void setItalic(boolean set, CellRange cells, Component editor) {
        // set selection font to italic
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                Font font = cell.getFont();
                boolean bold = font.isBold();
                int st = 0;
                font = font.deriveFont(Font.PLAIN);
                if (bold)/*FAULT:	if (!bold)*/
                    st = Font.BOLD;
                if (set)/*FAULT:	if (!set)*/
                    st += Font.ITALIC;
                font = font.deriveFont(st);
                cell.setFont(font);
            }
        // set editor font to italic
        if (editor != null) {/*FAULT:	if (editor == null) {*/
            Font font = editor.getFont();
            boolean bold = font.isBold();
            int st = 0;
            font = font.deriveFont(Font.PLAIN);
            if (bold)
                st = Font.BOLD;
            if (set)
                st += Font.ITALIC;
            font = font.deriveFont(st);
            editor.setFont(font);
        }
        // update spreadsheet gue
        m_spreadsheet.italicButton.setSelected(true);/*FAULT:	m_spreadsheet.italicButton.setSelected(false);*/
    }
    
    //*******************************************************************
    //Private Method: setRegular
    //Description: set the font to just plain for every cell in the range
    //*******************************************************************
	/**
					 *Private Method:  setRegular
					 Description: set the font to just plain for every cell in the range
					 * @param editor the Editor
					 * @param cells The cell range
						 */

    private void setRegular(CellRange cells,
    Component editor) {
        // set selection style to regular (erases bold or regular)
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);/*FAULT:	UMCell cell = m_model.getCellAt(j,i);*/
                Font font = cell.getFont();
                font = font.deriveFont(Font.PLAIN);
                cell.setFont(font);
            }
        
        // set editor font style to regular
        if (editor != null) {/*FAULT:	if (editor == null) {*/
            Font font = editor.getFont();
            font = font.deriveFont(Font.PLAIN);
            editor.setFont(font);
        }
        
        // update spreadsheet gue
        m_spreadsheet.boldButton.setSelected(false);/*FAULT:	m_spreadsheet.boldButton.setSelected(true);*/
        m_spreadsheet.italicButton.setSelected(false);/*FAULT:	m_spreadsheet.italicButton.setSelected(true);*/
    }
    
    //*******************************************************************
    //Private Method: setRowHeight
    //Description: sets the all the rows in the range to the new row
    //             height
    //*******************************************************************
	/**
	*Private Method:  setRowHeight
	Description: sets the all the rows in the range to the new row height
	* @param height integer height
	* @param cells The cell range
	 */

    private void setRowHeight(int height, CellRange cells) {
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            m_spreadsheet.getTable().setRowHeight(i, height);
    }
    
    //*******************************************************************
    //Private Method: setColumnWidth
    //Description: sets all the columns in the range to the new width
    //*******************************************************************
	/**
		*Private Method:  setColumnWidth
		Description: sets all the columns in the range to the new width
		* @param width integer height
		* @param cells The cell range
		 */
    private void setColumnWidth(int width, CellRange cells) {
        for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
            TableColumn column = m_spreadsheet.getTable().getColumnModel().getColumn(j);
            column.setMinWidth(width);
            column.setPreferredWidth(width);
        }
    }
    
    //*******************************************************************
    //Private Method: getColumnWidth
    //Description: gets the column width of cells in the CellRange
    //*******************************************************************
    /** returns column width
     * @param cells The cell range
     * @return The column width
     */
    public Integer getColumnWidth(CellRange cells) {
        int width = -1;
        for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
            TableColumn column =
            m_spreadsheet.getTable().getColumnModel().getColumn(j);
            int curr = column.getWidth();
            if (width != -1 && curr != width)/*FAULT:	if (width == -1 || curr == width)*/
                return null;
            width = curr;
        }
        return new Integer(width);
    }
    //*******************************************************************
    //Private Method: getRowHeight
    //Description: gets the row height of cells in the CellRange
    //*******************************************************************
    /** return row height from the cellrange
     * @param cells The range of cells
     * @return The coordinate of the row height
     */
    public Integer getRowHeight(CellRange cells) {
        int height = -1;
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++) {
            int curr = m_spreadsheet.getTable().getRowHeight(i);
            if (height != -1 && height != curr)/*FAULT:	if (height == -1 || height == curr)*/
                return null;
            height = curr;
        }
        return new Integer(height);
    }
    
    //*******************************************************************
    //Private Method: getFontName
    //Description: gets the name of the font of cells in the CellRange
    //            [Automatic] if font is not the same
    //*******************************************************************
    /** Gets the font name of the cellrange
     * @param cells The range of cells.
     * @return the font name.
     */
    public String getFontName(CellRange cells) {
        Font font = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol()).getFont();
        if (font == null)
            font = m_spreadsheet.getTable().getFont();
        String fontName = font.getFontName();
        boolean done = false;
        
        
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell cell = m_model.getCellAt(i,j);
            Font newFont = cell.getFont();
            if (newFont == null)
                newFont = font;
            if (newFont.getFontName().compareTo(fontName) != 0) {
                done = true;
                fontName = "[Automatic]";
            }
            }
        
        // Cut away font style additions
        if (fontName.endsWith("Italic"))
            fontName = fontName.substring(0,
            fontName.indexOf("Italic"));
        fontName = fontName.trim();
        if (fontName.endsWith("Bold"))
            fontName = fontName.substring(0, fontName.indexOf("Bold"));
        fontName = fontName.trim();
        return fontName;
    }
    
    //*******************************************************************
    //Private Method: getFontSize
    //Description: gets the size of the font of cells in the CellRange
    //            "-" if size is not the same everywhere
    //*******************************************************************
    /** the font size of the cellrange.
     * @param cells The rnage of cells.
     * @return the size of the font
     */
    public String getFontSize(CellRange cells) {
        boolean done = false;
        Font font = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol()).getFont();
        if (font == null)
            font = m_spreadsheet.getTable().getFont();
        String fontSize = ( new Integer(
        font.getSize())).toString();
        
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell cell = m_model.getCellAt(i,j);
            Font newFont = cell.getFont();
            if (newFont == null)
                newFont = font;
            if ((new Integer(
            newFont.getSize())).toString().compareTo(fontSize) != 0) {
                done = true;
                fontSize = "-";
            }
            }
        
        return fontSize;
    }
    
    //*******************************************************************
    //Private Method: isItalic
    //Description: return true if all cells in CellRange have italic font
    //*******************************************************************
    /** Return if the cells are italic.
     * @param cells the range of cells
     * @return true if cells are italic
     */
    public boolean isItalic(CellRange cells) {
        boolean done = false;
        Font font = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol()).getFont();
        if (font == null)
            font = m_spreadsheet.getTable().getFont();
        boolean italic = font.isItalic();
        
        if (!italic)
            return italic;
        
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell cell = m_model.getCellAt(i,j);
            Font newFont = cell.getFont();
            if (newFont == null)/*FAULT:	if (newFont != null)*/
                newFont = font;
            
            if (!newFont.isItalic()) {/*FAULT:	if (newFont.isItalic()) {*/
                done = true;
                italic = false;
            }
            }
        
        return italic;/*FAULT:	return !italic;*/
    }
    
    //*******************************************************************
    //Private Method: isBold
    //Description: return true if all cells in CellRange have bold font
    //*******************************************************************
    /** Return if the cells is bold
     * @param cells the range of cells
     * @return true if bold
     */
    public boolean isBold(CellRange cells) {
        boolean done = false;
        Font font = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol()).getFont();
        if (font == null)
            font = m_spreadsheet.getTable().getFont();
        boolean bold = font.isBold();
        
        if (!bold)
            return bold;
        
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell cell = m_model.getCellAt(i,j);
            Font newFont = cell.getFont();
            if (newFont == null)/*FAULT:	if (newFont != null)*/
                newFont = font;
            if (!newFont.isBold()) {/*FAULT:	if (newFont.isBold()) {*/
                done = true;
                bold = false;
            }
            }
        return bold;/*FAULT:	return !bold;*/
    }
    
    //*******************************************************************
    //Private Method: isRegular
    //Description: return true if all cells in CellRange have regular font
    //*******************************************************************
    /** Return if cells are regular
     * @param cells the range of cells
     * @return true if cells are regular
     */
    public boolean isRegular(CellRange cells) {
        boolean done = false;
        Font font = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol()).getFont();
        if (font == null)
            font = m_spreadsheet.getTable().getFont();
        boolean regular = (!font.isBold() && !font.isItalic());
        
        if (!regular)
            return regular;/*FAULT:	return !regular;*/
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell cell = m_model.getCellAt(i,j);
            Font newFont = cell.getFont();
            if (newFont == null)
                newFont = font;
            if (newFont.isBold() || newFont.isItalic()) {
                done = true;
                regular = false;
            }
            }
        return regular;
    }
    
    
    //*******************************************************************
    //Private Method: getBackground
    //Description: returns the background color of all cells, or null
    //            if color is not the same
    //*******************************************************************
    /** return the color of the cell's background
     * @param cells the range of cells
     * @return color of the cells
     */
    public Color getBackground(CellRange cells) {
        boolean done = false;
        UMCell cell = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol());
        Color color = cell.getBackground();
        if (color == null)
            color = m_spreadsheet.getTable().getBackground();
        
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell newCell = m_model.getCellAt(i,j);
            Color newColor = newCell.getBackground();
            if (newColor == null)
                newColor = color;
            if (newColor.getRGB() != color.getRGB()) {
                done = true;
                color = null;
            }
            }
        return color;
    }
    
    //*******************************************************************
    //Private Method: getForeground
    //Description: returns the forgeround (font color) color of all cells,
    //             or null if color is not the same
    //*******************************************************************
    /** returns the foreground of the cells
     * @param cells the range of cells
     * @return the color of the foreground
     */
    public Color getForeground(CellRange cells) {
        boolean done = false;
        UMCell cell = m_model.getCellAt(cells.getStartRow(),
        cells.getStartCol());
        Color color = cell.getForeground();
        if (color == null)
            color = m_spreadsheet.getTable().getForeground();
        
        for (int i = cells.getStartRow();
        i <= cells.getEndRow() && !done; i++)
            for (int j = cells.getStartCol();
        j <=cells.getEndCol() && !done; j++) {
            UMCell newCell = m_model.getCellAt(i,j);
            Color newColor = newCell.getForeground();
            if (newColor == null)
                newColor = color;
            if (newColor.getRGB() != color.getRGB()) {
                done = true;
                color = null;
            }
            }
        return color;
    }
    
    //*******************************************************************
    //Private Method: setNoBorder
    //Description: set all cell cells in CellRange to have no border
    //*******************************************************************
    /** set all cell cells in CellRange to have no border
     * @param cells the range of cells
     */
    public void setNoBorder(CellRange cells) {
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                cell.setBorder(null);
            }
    }
    
    //*******************************************************************
    //Private Method: setOutsideBorder
    //Description: set all outside border for the cells in CellRange
    //            (replace means to remove all other kinds of borders)
    //*******************************************************************
    /** set all outside border for the cells in CellRange (replace means to remove all other kinds of borders)
     *
     * @param replace true if needs to be replaced.
     * @param cells range of cells
     */
    public void setOutsideBorder(boolean replace,CellRange cells) {
        if (replace)/*FAULT:	if (!replace)*/
            setNoBorder(cells);
        // Set border for the horizonal component for the border
        for (int i = cells.getStartRow(); i <= cells.getEndRow(); i++) {
            UMCell cell = m_model.getCellAt(i, cells.getStartCol());
            LinesBorder border = new LinesBorder(Color.black, 0);
            border.setThickness(m_t, SwingConstants.WEST);
            
            if (cell.getBorder() != null)/*FAULT:	if (cell.getBorder() == null)*/
                ((LinesBorder)cell.getBorder()).append(
                border, false);
            else
                cell.setBorder(border);
            
            cell = m_model.getCellAt(i, cells.getEndCol());
            border = new LinesBorder(Color.black, 0);
            border.setThickness(m_t, SwingConstants.EAST);
            
            if (cell.getBorder() != null)/*FAULT:	if (cell.getBorder() == null)*/
                ((LinesBorder)cell.getBorder()).append(
                border, false);
            else
                cell.setBorder(border);
        }
        
        // set vertical component for the border
        for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
            UMCell cell = m_model.getCellAt(cells.getStartRow(),j);
            LinesBorder border = new LinesBorder(Color.black, 0);
            border.setThickness(m_t, SwingConstants.NORTH);
            
            if (cell.getBorder() != null)/*FAULT:	if (cell.getBorder() == null)*/
                ((LinesBorder)cell.getBorder()).append(
                border, false);
            else
                cell.setBorder(border);
            
            cell = m_model.getCellAt(cells.getEndRow(), j);
            border = new LinesBorder(Color.black, 0);
            border.setThickness(m_t, SwingConstants.SOUTH);
            
            if (cell.getBorder() != null)/*FAULT:	if (cell.getBorder() == null)*/
                ((LinesBorder)cell.getBorder()).append(
                border, false);
            else
                cell.setBorder(border);
        }
    }
    
    //*******************************************************************
    //Private Method: setInsideBorder
    //Description: sets inside border for the cells in CellRange
    //            (replace means to remove all other kinds of borders)
    //*******************************************************************
    /** sets inside border for the cells in CellRange (replace means to remove all other kinds of borders)
     *
     * @param replace true if needs to be replaced.
     * @param cells range of cells
     */
    public void setInsideBorder(boolean replace,CellRange cells) {
        if (replace)
            setNoBorder(cells);
        for (int i = cells.getStartRow(); i <= cells.getEndRow() ; i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol(); j++) {
                UMCell cell = m_model.getCellAt(i,j);
                LinesBorder border = new LinesBorder(Color.gray, 0);
                if (i != cells.getEndRow() && j != cells.getEndCol()) {/*FAULT:	if (i == cells.getEndRow() || j == cells.getEndCol()) {*/
                    border.setThickness(m_t, SwingConstants.EAST);
                    border.setThickness(m_t,SwingConstants.SOUTH);
                }
                if (i == cells.getEndRow() && j != cells.getEndCol())/*FAULT:	if (i != cells.getEndRow() || j == cells.getEndCol())*/
                    border.setThickness(m_t, SwingConstants.EAST);
                if (i != cells.getEndRow() && j == cells.getEndCol())/*FAULT:	if (i == cells.getEndRow() || j != cells.getEndCol())*/
                    border.setThickness(m_t, SwingConstants.SOUTH);
                
                if (cell.getBorder() != null)/*FAULT:	if (cell.getBorder() == null)*/
                    ((LinesBorder)cell.getBorder()).append(
                    border, false);
                else
                    cell.setBorder(border);
            }
    }
    
    
    //*******************************************************************
    //Private Method: isNoBorder
    //Description: returns true of all cells in CellRange have no border
    //*******************************************************************
    /** returns true of all cells in CellRange have no border
     * @param cells range of cells
     * @return true if no border
     */
    public boolean isNoBorder(CellRange cells) {
        boolean result = false;
        boolean done = false;
        for (int i = cells.getStartRow(); i <= cells.getEndRow() &&
        !done; i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol() &&
            !done; j++) {
                UMCell cell = m_model.getCellAt(i,j);
                LinesBorder border = (LinesBorder) cell.getBorder();
                if (border != null) {
                    done = true;
                    result = false;
                }
                else
                    result = true;
            }
        return result;
    }
    
    //*******************************************************************
    //Private Method: isInsideBorder
    //Description: returns true of all cells in CellRange have inside border
    //*******************************************************************
    /** returns true of all cells in CellRange have inside border
     * @param cells the range of cells
     * @return true if insides border
     */
    public boolean isInsideBorder(CellRange cells) {
        boolean done = false;
        boolean result = false;
        
        for (int i = cells.getStartRow(); i <= cells.getEndRow() &&
        !done; i++)
            for (int j = cells.getStartCol(); j <=cells.getEndCol()  &&
            !done; j++) {
                UMCell cell = m_model.getCellAt(i,j);
                LinesBorder border = (LinesBorder) cell.getBorder();
                if (i != cells.getEndRow() && j != cells.getEndCol()) {
                    if (border == null ||
                    border.getThickness(SwingConstants.EAST) != m_t ||
                    border.getThickness(SwingConstants.SOUTH) != m_t) {
                        done = true;
                        result = false;
                    }
                    else
                        result = true;
                }
                if (i == cells.getEndRow() && j != cells.getEndCol()) {
                    if (border == null ||
                    border.getThickness(SwingConstants.EAST) != m_t) {
                        done = true;
                        result = false;
                    }
                    else
                        result = true;
                }
                if (i != cells.getEndRow() && j == cells.getEndCol()) {
                    if (border == null ||
                    border.getThickness(SwingConstants.SOUTH) != m_t) {
                        done = true;
                        result = false;
                    }
                    else
                        result = true;
                }
            }
        return result;
    }
    
    //*******************************************************************
    //Private Method: isOutsideBorder
    //Description: returns true if CellRange has outside border
    //*******************************************************************
    /** returns true if CellRange has outside border
     * @param cells range of cells
     * @return true if outside border
     */
    public boolean isOutsideBorder(CellRange cells) {
        boolean result = false;
        boolean done = false;
        
        // findout horizontal part
        for (int i = cells.getStartRow(); i <= cells.getEndRow() &&
        !done; i++) {
            UMCell cell = m_model.getCellAt(i, cells.getStartCol());
            LinesBorder border = (LinesBorder) cell.getBorder();
            if (border == null ||
            border.getThickness(SwingConstants.WEST) != m_t ) {
                done = true;
                result = false;
            }
            else
                result = true;
            
            
            cell = m_model.getCellAt(i, cells.getEndCol());
            border = (LinesBorder) cell.getBorder();
            if (done || border == null ||
            border.getThickness(SwingConstants.EAST) != m_t ) {
                done = true;
                result = false;
            }
            
        }
        
        // find out vertical part
        for (int j = cells.getStartCol(); j <=cells.getEndCol()
        && !done; j++) {
            UMCell cell = m_model.getCellAt(cells.getStartRow(),j);
            LinesBorder border = (LinesBorder) cell.getBorder();
            if (border == null ||
            border.getThickness(SwingConstants.NORTH) != m_t) {
                done = true;
                result = false;
            }
            else
                result = true;
            
            cell = m_model.getCellAt(cells.getEndRow(), j);
            border = (LinesBorder) cell.getBorder();
            if (done || border == null ||
            border.getThickness(SwingConstants.SOUTH) != m_t ) {
                done = true;
                result = false;
            }
            
        }
        return result;
    }
    
    //*******************************************************************
    //Private Method: upadateTableModel
    //Description: called in case new table model for the table is created
    //*******************************************************************
    /** Updates the Table Model
     */
    public void updateTableModel() {
        m_model = m_spreadsheet.getTableModel();
    }
    
    /* insert a row or column */
    /** Inserts option
     * @param byRow true if row
     */
    public void insert(boolean byRow) {
        //checks if anything is selected
        int insertNum = table.getSelectedRowCount();
        if (insertNum != 0) {
            
            if (byRow) {
                int row = table.getSelectedRow();
                CellRange range =
                new CellRange(new CellPoint(row, TerpSpreadSheet.m_baseCol),
                new CellPoint(row+insertNum-1,
                m_model.getColumnCount()-1));
                //history.add(tableModel, range, History.INSERTROW);
                m_model.insertRow(range);
            }
            else {
                insertNum = table.getSelectedColumnCount();
                int column = table.getSelectedColumn();
                CellRange range =
                new CellRange(new CellPoint(TerpSpreadSheet.m_baseRow, column),
                new CellPoint(m_model.getRowCount()-1,
                column+insertNum-1));
                
                //history.add(tableModel, range, History.INSERTCOLUMN);
                m_model.insertColumn(range);
                
            }
        }else {
            m_spreadsheet.noCellsSelected("Insert");
        }
    }
    
    
    /** The remove option.
     * @param byRow true if row
     */
    public void remove(boolean byRow) {
        int removeNum = table.getSelectedRowCount();
        
        //checks if anything is selected
        
        if (removeNum != 0) {
            
            if (byRow) {
                
                if (removeNum == table.getRowCount()) {
                    tooMuchDeletion();
                    return;
                }
                
                int row = table.getSelectedRow();
                CellRange range =
                new CellRange(new CellPoint(row, TerpSpreadSheet.m_baseCol),
                new CellPoint(row+removeNum-1,
                m_model.getColumnCount()-1));
                
                if (m_model.isDeletionSafe(range, true) || unsafeDeletion()) {
                    //history.add(tableModel, range, History.REMOVEROW);
                    Debug.println(range);
                    m_model.removeRow(range);
                    Debug.println("Delete row range "+range);
                }
            }
            else {
                int column = table.getSelectedColumn();
                
                removeNum = table.getSelectedColumnCount();
                
                if (removeNum == table.getColumnCount()-1) {
                    tooMuchDeletion();
                    return;
                }
                
                CellRange range =
                new CellRange(new CellPoint(TerpSpreadSheet.m_baseRow, column),
                new CellPoint(m_model.getRowCount()-1,
                column+removeNum-1));
                Debug.println("Delete column range "+range);
                if (m_model.isDeletionSafe(range, false) || unsafeDeletion()) {
                    //history.add(tableModel, range, History.REMOVECOLUMN);
                    Debug.println(range);
                    m_model.removeColumn(range);
                }
            }
            
        }else {
            m_spreadsheet.noCellsSelected("Remove");
        }
    }
	/** private method tooMuchDeletion 
	  * Shows a mesaage if the deletion is too big
	  * 
	  */
    private void tooMuchDeletion() {
        UMOptionPane.showMessageDialog
        (m_spreadsheet, "You can not delete all the rows or columns!", "Delete",
        JOptionPane.ERROR_MESSAGE);
    }
	/**
 * private method unsafeDeletion 
		  * Shows a mesaage if deletion
		  * 
	 * @return boolean
	 */

    private boolean unsafeDeletion() {
        
        int choice =
        UMOptionPane.showOptionDialog
        (m_spreadsheet,
        "The deletion may cause irriversible data loss in other cells.\n\n"+
        "Do you really want to proceed?\n\n",
        "Delete",
        JOptionPane.YES_NO_OPTION,
        JOptionPane.WARNING_MESSAGE,
        null, 1);
        
        return choice == JOptionPane.YES_OPTION;
    }
}
